from dataclasses import dataclass

from dnsimple.struct import Struct


@dataclass
class Dnssec(Struct):
    """
    Represents a DNSSEC for a Domain

    See https://developer.dnsimple.com/v2/domains/dnssec/
    """

    enabled = None
    """True if DNSSEC is enabled on the domain, otherwise false"""
    created_at = None
    """When the DNSSEC was created in DNSimple"""
    updated_at = None
    """When the DNSSEC was last updated in DNSimple"""

    def __init__(self, data):
        super().__init__(data)



from dataclasses import dataclass

from dnsimple.struct import Struct


@dataclass
class Collaborator(Struct):
    """
    Represents a collaborator for a domain in the account

    See: https://developer.dnsimple.com/v2/domains/collaborators/
    """

    id = None
    """The collaborator ID in DNSimple"""
    domain_id = None
    """The associated domain ID"""
    domain_name = None
    """The associated domain name"""
    user_id = None
    """The user ID (if the collaborator accepted the invitation)."""
    user_email = None
    """The user email"""
    invitation = None
    """Invitation"""
    created_at = None
    """When the collaborator was created in DNSimple"""
    updated_at = None
    """When the collaborator was last updated in DNSimple"""
    accepted_at = None
    """When the collaborator accepted the invitation"""

    def __init__(self, data):
        super().__init__(data)



from dnsimple.struct.struct import Struct
from dnsimple.struct.access_token import AccessToken
from dnsimple.struct.account import Account
from dnsimple.struct.certificate import Certificate, CertificateBundle, LetsencryptCertificateInput, LetsencryptCertificateRenewalInput, CertificatePurchase, CertificateRenewal
from dnsimple.struct.charge import Charge, ChargeItem
from dnsimple.struct.collaborator import Collaborator
from dnsimple.struct.contact import Contact
from dnsimple.struct.dnssec import Dnssec
from dnsimple.struct.delegation_signer_record import DelegationSignerRecord, DelegationSignerRecordInput
from dnsimple.struct.domain import Domain
from dnsimple.struct.domain_check import DomainCheck
from dnsimple.struct.domain_premium_price import DomainPremiumPrice, DomainPremiumPriceOptions
from dnsimple.struct.domain_price import DomainPrice
from dnsimple.struct.domain_registration import DomainRegistration, DomainRegistrationRequest
from dnsimple.struct.domain_renewal import DomainRenewal, DomainRenewRequest
from dnsimple.struct.domain_restore import DomainRestore, DomainRestoreRequest
from dnsimple.struct.domain_transfer import DomainTransfer, DomainTransferRequest
from dnsimple.struct.domain_transfer_lock import DomainTransferLock
from dnsimple.struct.domain_push import DomainPush
from dnsimple.struct.dns_analytics import DnsAnalytics
from dnsimple.struct.email_forward import EmailForward, EmailForwardInput
from dnsimple.struct.registrant import RegistrantChange, CheckRegistrantChangeInput, CreateRegistrantChangeInput, RegistrantChangeCheck
from dnsimple.struct.service import Service
from dnsimple.struct.template import Template
from dnsimple.struct.template_record import TemplateRecord
from dnsimple.struct.tld import Tld, TldExtendedAttribute, TldExtendedAttributeOption
from dnsimple.struct.user import User
from dnsimple.struct.vanity_name_server import VanityNameServer
from dnsimple.struct.webhook import Webhook
from dnsimple.struct.whoami import Whoami
from dnsimple.struct.whois_privacy import WhoisPrivacy, WhoisPrivacyRenewal
from dnsimple.struct.zone import Zone
from dnsimple.struct.zone_distribution import ZoneDistribution
from dnsimple.struct.zone_file import ZoneFile
from dnsimple.struct.zone_record import ZoneRecord, ZoneRecordInput, ZoneRecordUpdateInput
from dnsimple.struct.batch_change_zone_records import (
    BatchChangeZoneRecordsInput,
    BatchChangeZoneRecordsCreateInput,
    BatchChangeZoneRecordsUpdateInput,
    BatchChangeZoneRecordsDeleteInput,
    BatchChangeZoneRecordsResponse,
    BatchChangeZoneRecordsDeleteResponse
)



from dnsimple.response import Response
from dnsimple.struct import Tld, TldExtendedAttribute


class Tlds(object):
    """Handles communication with the Tld related methods of the DNSimple API."""

    def __init__(self, client):
        self.client = client

    def list_tlds(self, sort=None):
        """
        Lists the TLDs available for registration

        See https://developer.dnsimple.com/v2/tlds/#listTlds

        :param sort: str
            Comma separated key-value pairs: the name of a field and the order criteria (asc for ascending and desc for
            descending).

            Possible sort criteria:
                - tld: Sort TLDs by tld (i.e. 'tld:asc')

        :return: dnsimple.Response
            The tld list
        """
        response = self.client.get('/tlds', sort=sort)
        return Response(response, Tld)

    def get_tld(self, tld):
        """
        Gets the details of a TLD

        See https://developer.dnsimple.com/v2/tlds/#getTld
        
        :param tld: str
            The TLD name

        :return: dnsimple.Response
            The TLD details
        """
        response = self.client.get(f'/tlds/{tld}')
        return Response(response, Tld)

    def get_tld_extended_attributes(self, tld):
        """
        Gets the extended attributes for a TLD

        See https://developer.dnsimple.com/v2/tlds/#getTldExtendedAttributes

        :param tld: str
            The TLD name

        :return: dnsimple.Response
            The TLDs extended attributes
        """
        response = self.client.get(f'/tlds/{tld}/extended_attributes')
        return Response(response, TldExtendedAttribute)



from dataclasses import dataclass

from dnsimple.struct import Struct


@dataclass
class EmailForward(Struct):
    id = None
    """The email forward ID in DNSimple"""
    domain_id = None
    """The associated domain ID"""
    alias_email = None
    destination_email = None
    active = None
    created_at = None
    """When the email forward was created in DNSimple"""
    updated_at = None
    """When the email forward was last updated in DNSimple"""

    def __init__(self, data):
        super().__init__(data)
        setattr(self, 'email_from', data['alias_email'])
        setattr(self, 'email_to', data['destination_email'])


@dataclass
class EmailForwardInput(object):
    def __init__(self, alias_name, destination_email):
        self.alias_name = alias_name
        self.destination_email = destination_email




from dataclasses import dataclass

from dnsimple.struct import Struct


@dataclass
class Account(Struct):
    """
    Represents an Account.

    See https://developer.dnsimple.com/v2/identity/
    """

    id = None
    """The account ID in DNSimple"""
    email = None
    """The account email"""
    plan_identifier = None
    """The identifier of the plan the account is subscribed to"""
    created_at = None
    """When the account was created in DNSimple"""
    updated_at = None
    """When the account was last updated in DNSimple"""

    def __init__(self, data):
        super().__init__(data)



from dnsimple.response import Response
from dnsimple.struct import Certificate, CertificateBundle, CertificatePurchase, CertificateRenewal, LetsencryptCertificateInput, LetsencryptCertificateRenewalInput


class Certificates(object):
    """The Certificates service handles communication with the certificate related methods of the DNSimple API"""

    def __init__(self, client):
        self.client = client

    def list_certificates(self, account_id, domain, page=None, per_page=None):
        """
        List the certificates for a domain in the account.

        See https://developer.dnsimple.com/v2/certificates/#listCertificates

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id
        :param page: int
            The page to return (default: 1)
        :param per_page: int
            The number of entries to return per page (default: 30, maximum: 100)

        :return: dnsimple.Response
            The list of certificates for the domain
        """
        response = self.client.get(f'/{account_id}/domains/{domain}/certificates', page=page, per_page=per_page)
        return Response(response, Certificate)

    def get_certificate(self, account_id, domain, certificate_id):
        """
        Get the details of a certificate.

        See https://developer.dnsimple.com/v2/certificates/#getCertificate

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id
        :param certificate_id: int
            The certificate id

        :return: dnsimple.Response
            The certificate requested
        """
        response = self.client.get(f'/{account_id}/domains/{domain}/certificates/{certificate_id}')
        return Response(response, Certificate)

    def download_certificate(self, account_id, domain, certificate_id):
        """
        Gets the PEM-encoded certificate, along with the root certificate and intermediate chain

        See https://developer.dnsimple.com/v2/certificates/#downloadCertificate

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id
        :param certificate_id: int
            The certificate id

        :return: dnsimple.Response
            The certificate in the domain
        """
        response = self.client.get(f'/{account_id}/domains/{domain}/certificates/{certificate_id}/download')
        return Response(response, CertificateBundle)

    def get_certificate_private_key(self, account_id, domain, certificate_id):
        """
        Get the PEM-encoded certificate private key

        See https://developer.dnsimple.com/v2/certificates/#getCertificatePrivateKey

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id
        :param certificate_id: int
            The certificate ID

        :return: dnsimple.Response
            The PEM-encoded certificate private key
        """
        response = self.client.get(f'/{account_id}/domains/{domain}/certificates/{certificate_id}/private_key')
        return Response(response, CertificateBundle)

    def purchase_letsencrypt_certificate(self, account_id, domain, certificate_input = LetsencryptCertificateInput()):
        """
        Purchase a Let's Encrypt certificate

        This method creates a new certificate order. The certificate ID should be used to request the issuance of the
        certificate using {#issue_letsencrypt_certificate}.

        See https://developer.dnsimple.com/v2/certificates/#purchaseLetsencryptCertificate

        :param account_id: int
            The account id
        :param domain: int/str
            The domain name or id
        :param certificate_input: dnsimple.struct.LetsencryptCertificateInput
            A set of attributes to purchase the Let's Encrypt certificate

        :return: dnsimple.Response
            The certificate purchase
        """
        response = self.client.post(f'/{account_id}/domains/{domain}/certificates/letsencrypt',
                                    data=certificate_input.to_json())
        return Response(response, CertificatePurchase)

    def issue_letsencrypt_certificate(self, account_id, domain, certificate_id):
        """
        Issue a pending Let's Encrypt certificate order.

        Note that the issuance process is async. A successful response means the issuance request has been successfully
        acknowledged and queued for processing.

        See https://developer.dnsimple.com/v2/certificates/#issueLetsencryptCertificate

        :param account_id: int
            The account id
        :param domain: int/str
            The domain name or id
        :param certificate_id: int
            The certificate id

        :return: dnsimple.Response
            The certificate issued
        """
        response = self.client.post(f'/{account_id}/domains/{domain}/certificates/letsencrypt/{certificate_id}/issue')
        return Response(response, Certificate)

    def purchase_letsencrypt_certificate_renewal(self, account_id, domain, certificate_id, certificate_renewal_input = LetsencryptCertificateRenewalInput()):
        """
        Purchase a Let's Encrypt certificate renewal.

        See https://developer.dnsimple.com/v2/certificates/#purchaseRenewalLetsencryptCertificate

        :param account_id: int
            The account id
        :param domain: int/str
            The domain name or id
        :param certificate_id: int
            The certificate id

        :return: dnsimple.Response
            The certificate renewal
        """
        response = self.client.post(f'/{account_id}/domains/{domain}/certificates/letsencrypt/{certificate_id}/renewals', data=certificate_renewal_input.to_json())
        return Response(response, CertificateRenewal)

    def issue_letsencrypt_certificate_renewal(self, account_id, domain, certificate_id, certificate_renewal_id):
        """
        Issue a pending Let's Encrypt certificate renewal order.

        Note that the issuance process is async. A successful response means the issuance request has been
        successfully acknowledged and queued for processing.

        See https://developer.dnsimple.com/v2/certificates/#issueRenewalLetsencryptCertificate

        :param account_id: int
            The account id
        :param domain: int/str
            The domain name or id
        :param certificate_id: int
            The certificate id
        :param certificate_renewal_id: int
            The certificate renewal id

        :return: dnsimple.Response
            The certificate to be renewed
        """
        response = self.client.post(f'/{account_id}/domains/{domain}/certificates/letsencrypt/{certificate_id}/renewals/{certificate_renewal_id}/issue')
        return Response(response, Certificate)



from dataclasses import dataclass

from dnsimple.struct import Struct


@dataclass
class ZoneDistribution(Struct):
    """Represents the zone distribution"""

    distributed = None
    """true if the zone is properly distributed across all DNSimple name servers."""

    def __init__(self, data):
        super().__init__(data)



import json
from dataclasses import dataclass

import omitempty

from dnsimple.struct import Struct


class DomainRestoreRequest(dict):
    """DomainRestoreRequest represents the attributes you can pass to a restore API request."""
    def __init__(self, premium_price=None):
        dict.__init__(self, premium_price=premium_price)

    def to_json(self):
        return json.dumps(omitempty(self))


@dataclass
class DomainRestore(Struct):

    """Represents the result of a domain restore call."""
    id = None
    """The domain registration ID in DNSimple"""
    domain_id = None
    """The associated domain ID"""
    state = None
    """The state of the restore"""
    created_at = None
    """When the domain restore was created in DNSimple"""
    updated_at = None
    """When the domain restore was last updated in DNSimple"""

    def __init__(self, data):
        super().__init__(data)



import json
from dataclasses import dataclass

import omitempty

from dnsimple.struct import Struct


@dataclass
class DelegationSignerRecord(Struct):
    """Represents a delegation signer record"""

    id = None
    """The ID of the delegation signer record in DNSimple."""
    domain_id = None
    """The associated domain ID."""
    algorithm = None
    """The signing algorithm used."""
    digest = None
    """The digest value."""
    digest_type = None
    """The digest type used."""
    keytag = None
    """The keytag for the associated DNSKEY."""
    public_key = None
    """The public key that references the corresponding DNSKEY record."""
    created_at = None
    """When the delegation signing record was created in DNSimple."""
    updated_at = None
    """When the delegation signing record was last updated in DNSimple."""

    def __init__(self, data):
        super().__init__(data)


@dataclass
class DelegationSignerRecordInput(dict):
    """Represents the input we send to create a domain delegation signer record"""

    def __init__(self, algorithm, digest=None, digest_type=None, keytag=None, public_key=None):
        """
        :param algorithm: str
            Required DNSSEC algorithms defined in
            http://www.iana.org/assignments/dns-sec-alg-numbers/dns-sec-alg-numbers.xhtml
            - pass the Number value as a string (i.e. '8').
        :param digest: str
            Required if TLD requires DS data.
            The hexadecimal representation of the digest of the corresponding DNSKEY record.
        :param digest_type: str
            Required if TLD requires DS data.
            DNSSEC digest types defined in http://www.iana.org/assignments/ds-rr-types/ds-rr-types.xhtml
            - pass the Number value as string (i.e. '2').
        :param keytag: str
            Required if TLD requires DS data.
            A keytag that references the corresponding DNSKEY record.
        :param public_key: str
            Required if TLD requires KEY data.
            The public key that references the corresponding DNSKEY record.
        """
        dict.__init__(self, algorithm=algorithm, digest=digest, digest_type=digest_type, keytag=keytag, public_key=public_key)

    def to_json(self):
        return json.dumps(omitempty(self))



from urllib.parse import urljoin

import urllib.parse

from dnsimple.response import Response
from dnsimple.struct import AccessToken


class Oauth(object):
    def __init__(self, client):
        self.client = client

    def exchange_authorization_for_token(self, code, client_id, client_secret, state, redirect_uri):
        """
        Exchange the short-lived authorization code for an access token
        you can use to authenticate your API calls.

        See https://developer.dnsimple.com/v2/oauth/

        :param code: str
            The authorization code, which can be used in the next step to retrieve a bearer token.
        :param client_id: str
            Client Id you received when the application was registered with DNSimple.
        :param client_secret: str
            Client Secret you received when the application was registered with DNSimple.
        :param state: str
            An unguessable random string. It is used to protect against cross-site request forgery
            attacks and it will be passed back to your redirect URI.
        :param redirect_uri: str
            Where to redirect the user after authorization has completed. This must be the exact
            URI registered or a subdirectory.
        :return: OAuthToken
            The OAuthToken object containing the access token to be used in subsequent calls to the API
        """
        response = self.client.post('/oauth/access_token', data={'code': code, 'client_id': client_id,
                                                                 'client_secret': client_secret, 'state': state,
                                                                 'redirect_uri': redirect_uri,
                                                                 'grant_type': 'authorization_code'})
        return Response(response, AccessToken)

    def authorize_url(self, client_id, redirect_uri=None, state=None, scope=None):
        """
        Generates the URL to authorize an user for an application via the OAuth2 flow.

        :param client_id: str
            The service ID you received from DNSimple when you registered the application.
        :param redirect_uri: str
            Only used to validate that it matches the original /oauth/authorize, not used to redirect again.
        :param state: str
            The state content originally passed to /oauth/authorize.
        :param scope: str
            The scopes to request from the user.
        :return: str
            The URL to redirect the user to authorize.
        """
        base_url = f'{self.client.base_url}/oauth/authorize?client_id={client_id}&response_type=code'
        query_params = {}

        if redirect_uri is not None:
            query_params['redirect_url'] = redirect_uri
        if state is not None:
            query_params['state'] = state
        if scope is not None:
            query_params['scope'] = scope

        if query_params:
            return f'{base_url}&{urllib.parse.urlencode(query_params)}'

        return base_url



import json
from dataclasses import dataclass

import omitempty
import warnings

from dnsimple.struct import Struct


@dataclass
class Certificate(Struct):
    """Represents a Certificate in DNSimple"""

    id = None
    """The certificate ID in DNSimple"""
    domain_id = None
    """The associated domain ID"""
    contact_id = None
    """The associated contact ID"""
    name = None
    """The certificate name"""
    common_name = None
    """The certificate common name"""
    years = None
    """The years the certificate will last"""
    csr = None
    """The certificate CSR"""
    state = None
    """The certificate state"""
    auto_renew = False
    """True if the certificate is set to auto-renew on expiration"""
    alternate_names = None
    """The certificate alternate names"""
    authority_identifier = None
    """The Certificate Authority (CA) that issued the certificate"""
    created_at = None
    """When the certificate was created in DNSimple"""
    updated_at = None
    """When the certificate was last updated in DNSimple"""
    expires_on = None
    """When the certificate will expire"""

    def __init__(self, data):
        super().__init__(data)


@dataclass
class CertificateBundle(Struct):
    """Represents a certificate download in DNSimple"""
    server = None
    """The server certificate"""
    root = None
    """The root certificate"""
    chain = None
    """The intermediate certificates"""
    private_key = None
    """The certificate private key"""

    def __init__(self, data):
        super().__init__(data)


class LetsencryptCertificateInput(dict):
    def __init__(self, contact_id=None, auto_renew=None, name=None, alternate_names=None, signature_algorithm=None):
        dict.__init__(self, auto_renew=auto_renew, name=name, alternate_names=alternate_names, signature_algorithm=signature_algorithm)
        if contact_id is not None:
            warnings.warn("DEPRECATION WARNING: LetsencryptCertificateInput#contact_id is deprecated and its value is ignored and will be removed in the next major version.")


    def to_json(self):
        return json.dumps(omitempty(self))


class LetsencryptCertificateRenewalInput(dict):
    def __init__(self, auto_renew=None, signature_algorithm=None):
        dict.__init__(self, auto_renew=auto_renew, signature_algorithm=signature_algorithm)


    def to_json(self):
        return json.dumps(omitempty(self))


@dataclass
class CertificatePurchase(Struct):
    """Represents a certificate purchase in DNSimple"""
    id = None
    """The certificate purchase ID in DNSimple"""
    certificate_id = None
    """The certificate ID"""
    state = None
    """The certificate renewal state"""
    auto_renew = None
    """True if the certificate is requested to auto-renew"""
    created_at = None
    """When the certificate renewal was created in DNSimple"""
    updated_at = None
    """When the certificate renewal was last updated in DNSimple"""

    def __init__(self, data):
        super().__init__(data)


@dataclass
class CertificateRenewal(Struct):
    """Represents a Certificate Renewal in DNSimple"""

    id = None
    """The certificate renewal ID in DNSimple"""
    old_certificate_id = None
    """The old certificate ID"""
    new_certificate_id = None
    """The new certificate ID"""
    state = None
    """The certificate renewal state"""
    auto_renew = None
    """True if the certificate is requested to auto-renew"""
    created_at = None
    """When the certificate renewal was created in DNSimple"""
    updated_at = None
    """When the certificate renewal was last updated in DNSimple"""

    def __init__(self, data):
        super().__init__(data)



from dataclasses import dataclass

from dnsimple.struct import Struct


@dataclass
class DomainCheck(Struct):
    domain = None
    """The domain name"""
    available = False
    """True if the domain is available"""
    premium = False
    """True if the domain is a premium domain"""

    def __init__(self, data):
        super().__init__(data)



from dataclasses import dataclass

from dnsimple.struct import Struct


@dataclass
class VanityNameServer(Struct):
    id = None
    """The vanity name server ID in DNSimple"""
    name = None
    """The vanity name server name"""
    ipv4 = None
    """The vanity name server IPv4"""
    ipv6 = None
    """The vanity name server IPv6"""
    created_at = None
    """When the vanity name server was created in DNSimple"""
    updated_at = None
    """When the vanity name server was last updated in DNSimple"""

    def __init__(self, data):
        super().__init__(data)



import json
import warnings

from dnsimple.response import Response
from dnsimple.struct import Domain, Dnssec, Collaborator, DelegationSignerRecord, EmailForward, DomainPush


class Domains(object):
    """
    The Domains Service handles the domains endpoint of the DNSimple API.

    See https://developer.dnsimple.com/v2/domains
    """

    def __init__(self, client):
        self.client = client

    def list_domains(self, account_id, sort=None, filter=None, page=None, per_page=None):
        """
        Lists the domains in the account.

        See https://developer.dnsimple.com/v2/domains/#list

        :param account_id: int
            The account ID
        :param sort: str
            Comma separated key-value pairs: the name of a field and the order criteria (asc for ascending and desc for
            descending).

            Possible sort criteria:
                - id: Sort domains by ID (i.e. 'id:asc')
                - name: Sort domains by name (alphabetical order) (i.e. 'name:desc')
                - expiration: Sort domains by expiration date (i.e. 'expiration:asc')
        :param filter: dict
            Makes it possible to ask only for the exact subset of data that you you’re looking for.

            Possible filters:
                - name_like: Only include domains containing given string (i.e. {'name_like': 'example.com'} )
                - registrant_id: Only include domains containing given registrant ID (i.e. {'registrant_id': 1010} )
        :param page: int
            The page to return (default: 1)
        :param per_page: int
            The number of entries to return per page (default: 30, maximum: 100)

        :return: dnsimple.Response
            A list of domains
        """
        response = self.client.get(f'/{account_id}/domains', sort=sort, filter=filter, page=page, per_page=per_page)
        return Response(response, Domain)

    def create_domain(self, account_id, domain_name):
        """
        Creates a domain in the account.

        See https://developer.dnsimple.com/v2/domains/#create

        :param account_id: int
            The account ID
        :param domain_name: str
            The name of the domain
        :return: dnsimple.Response
            The newly created domain
        """
        response = self.client.post(f'/{account_id}/domains', data=json.dumps({'name': domain_name}))
        return Response(response, Domain)

    def get_domain(self, account_id, domain):
        """
        Retrieves the details of an existing domain.

        See https://developer.dnsimple.com/v2/domains/#getDomain

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or ID
        :return: dnsimple.Response
            The domain requested
        """
        response = self.client.get(f'/{account_id}/domains/{domain}')
        return Response(response, Domain)

    def delete_domain(self, account_id, domain):
        """
        Permanently deletes a domain from the account.

        For domains which are registered with DNSimple, this will not delete the domain from the registry,
        nor perform a refund.

        See https://developer.dnsimple.com/v2/domains/#deleteDomain

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or ID
        :return: dnsimple.Response
            An empty response
        """
        response = self.client.delete(f'/{account_id}/domains/{domain}')
        return Response(response)

    def enable_dnssec(self, account_id, domain):
        """
        Enable DNSSEC for the domain in the account. This will sign the zone. If the domain is registered it will also
        add the DS record to the corresponding registry.

        See https://developer.dnsimple.com/v2/domains/dnssec/#enableDomainDnssec

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id

        :return: dnsimple.Response
            The DNSSEC status
        """
        response = self.client.post(f'/{account_id}/domains/{domain}/dnssec')
        return Response(response, Dnssec)

    def disable_dnssec(self, account_id, domain):
        """
        Disable DNSSEC for the domain in the account.

        See https://developer.dnsimple.com/v2/domains/dnssec/#disableDomainDnssec

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id

        :return: dnsimple.Response
            An empty response
        """
        response = self.client.delete(f'/{account_id}/domains/{domain}/dnssec')
        return Response(response)

    def get_dnssec(self, account_id, domain):
        """
        Get the status of DNSSEC, indicating whether it is currently enabled or disabled.

        See https://developer.dnsimple.com/v2/domains/dnssec/#getDomainDnssec

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id

        :return: dnsimple.Response
            The DNSSEC status requested
        """
        response = self.client.get(f'/{account_id}/domains/{domain}/dnssec')
        return Response(response, Dnssec)

    def list_domain_delegation_signer_records(self, account_id, domain, sort=None, page=None, per_page=None):
        """
        List delegation signer records for the domain in the account.

        See https://developer.dnsimple.com/v2/domains/dnssec/#listDomainDelegationSignerRecords

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id

        :param sort: str
            Comma separated key-value pairs: the name of a field and the order criteria (asc for ascending and desc for
            descending).

            Possible sort criteria:
                - id: Sort delegation signer records by ID (i.e. 'id:asc')
                - created_at: Sort delegation signer records by creation date (i.e. 'created_at:asc')

        :param page: int
            The page to return (default: 1)
        :param per_page: int
            The number of entries to return per page (default: 30, maximum: 100)

        :return: dnsimple.Response
            A list of delegation signer records for the domain in the account
        """
        response = self.client.get(f'/{account_id}/domains/{domain}/ds_records', sort=sort, page=page, per_page=per_page)
        return Response(response, DelegationSignerRecord)

    def create_domain_delegation_signer_record(self, account_id, domain, ds_input):
        """
        Create a delegation signer record

        You only need to create a delegation signer record manually if your domain is registered with DNSimple but
        hosted with another DNS provider that is signing your zone. To enable DNSSEC on a domain that is hosted with
        DNSimple, use the DNSSEC enable endpoint.

        See https://developer.dnsimple.com/v2/domains/dnssec/#createDomainDelegationSignerRecord

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id
        :param ds_input: DelegationsSignerRecordInput
            The delegation signer record input to create the delegation signer record

        :return: dnsimple.Response
            The newly created domain delegation signer record
        """
        response = self.client.post(f'/{account_id}/domains/{domain}/ds_records', data=ds_input.to_json())
        return Response(response, DelegationSignerRecord)

    def get_delegation_signer_record(self, account_id, domain, ds_record_id):
        """
        Get the delegation signer record under the domain in the account

        See https://developer.dnsimple.com/v2/domains/dnssec/#getDomainDelegationSignerRecord

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id
        :param ds_record_id: int
            The delegation signer record id

        :return: dnsimple.Response
            The domain delegation signer record requested
        """
        response = self.client.get(f'/{account_id}/domains/{domain}/ds_records/{ds_record_id}')
        return Response(response, DelegationSignerRecord)

    def delete_domain_delegation_signer_record(self, account_id, domain, ds_record_id):
        """
        Delete the delegation signer record under the domain in the account

        See https://developer.dnsimple.com/v2/domains/dnssec/#deleteDomainDelegationSignerRecord

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id
        :param ds_record_id: int
            The delegation signer record id

        :return: dnsimple.Response
            An empty response
        """
        response = self.client.delete(f'/{account_id}/domains/{domain}/ds_records/{ds_record_id}')
        return Response(response)

    def list_email_forwards(self, account_id, domain, sort=None, page=None, per_page=None):
        """
        List email forwards for the domain in the account.

        See https://developer.dnsimple.com/v2/domains/email-forwards/#listEmailForwards

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id
        :param sort: str
            Comma separated key-value pairs: the name of a field and the order criteria (asc for ascending and desc for
            descending).

            Possible sort criteria:
                - id: Sort email forwards by ID (i.e. 'id:asc')
                - from: Sort email forwards by from_email (aka from in alphabetical order) (i.e. 'from:desc')
                - to: Sort email forwards by to_email (aka to in alphabetical order) (i.e. 'to:asc')

        :param page: int
            The page to return (default: 1)
        :param per_page: int
            The number of entries to return per page (default: 30, maximum: 100)

        :return: dnsimple.Response
            The list of email forwards for the domain in the account
        """
        response = self.client.get(f'/{account_id}/domains/{domain}/email_forwards', sort=sort, page=page, per_page=per_page)
        return Response(response, EmailForward)

    def create_email_forward(self, account_id, domain, email_forwards_input):
        """
        Create an email forward under the domain in the account

        See https://developer.dnsimple.com/v2/domains/email-forwards/#createEmailForward

        :param account_id: int
            The account Id
        :param domain: int/str
            The domain name or id
        :param email_forwards_input:
            The email forwards input

        :return: dnsimple.Response
            The newly created email forward under the domain in the account
        """
        response = self.client.post(f'/{account_id}/domains/{domain}/email_forwards', data=json.dumps({'alias_name': email_forwards_input.alias_name, 'destination_email': email_forwards_input.destination_email}))
        return Response(response, EmailForward)

    def get_email_forward(self, account_id, domain, email_forward_id):
        """
        Get the email forward in the domain in the account

        See https://developer.dnsimple.com/v2/domains/email-forwards/#getEmailForward

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id
        :param email_forward_id: int
            The email forward id

        :return: dnsimple.Response
            The email forward requested
        """
        response = self.client.get(f'/{account_id}/domains/{domain}/email_forwards/{email_forward_id}')
        return Response(response, EmailForward)

    def delete_email_forward(self, account_id, domain, email_forward_id):
        """
        Delete the email forward from the domain.

        See https://developer.dnsimple.com/v2/domains/email-forwards/#deleteEmailForward

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id
        :param email_forward_id: int
            The email forward id

        :return: dnsimple.Response
            An empty response
        """
        response = self.client.delete(f'/{account_id}/domains/{domain}/email_forwards/{email_forward_id}')
        return Response(response)

    def initiate_push(self, account_id, domain, push_input):
        """
        Initiate a push for the domain.

        See https://developer.dnsimple.com/v2/domains/pushes/#initiate

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id
        :param push_input: dnsimple.struct.domain_push.DomainPushInput
            The data to send to initiate the push

        :return: dnsimple.Response
            The newly created domain push
        """
        response = self.client.post(f'/{account_id}/domains/{domain}/pushes', data=push_input.to_json())
        return Response(response, DomainPush)

    def list_pushes(self, account_id, page=None, per_page=None):
        """
       List pending pushes for the target account..

        See https://developer.dnsimple.com/v2/domains/pushes/#listPushes

        :param account_id: int
            The account ID
        :param page: int
            The page to return (default: 1)
        :param per_page: int
            The number of entries to return per page (default: 30, maximum: 100)

        :return: dnsimple.Response
            The list of pushes for the domain
        """
        response = self.client.get(f'/{account_id}/pushes', page=page, per_page=per_page)
        return Response(response, DomainPush)

    def accept_push(self, account_id, push_id, push_input):
        """
        Accept a push for the target account

        See https://developer.dnsimple.com/v2/domains/pushes/#acceptPush

        :param account_id: int
            The account ID
        :param push_id: int
            The push ID
        :param push_input: dnsimple.struct.domain_push.DomainPushInput
            The data to send to accept the push

        :return: dnsimple.Response
            An empty response
        """
        response = self.client.post(f'/{account_id}/pushes/{push_id}', data=push_input.to_json())
        return Response(response)

    def reject_push(self, account_id, push_id):
        """
        Reject a push for the target account

        See: https://developer.dnsimple.com/v2/domains/pushes/#rejectPush

        :param account_id: int
            The account ID
        :param push_id: int
            Then push ID

        :return: dnsimple.Response
            An empty response
        """
        response = self.client.delete(f'/{account_id}/pushes/{push_id}')
        return Response(response)



from requests.auth import AuthBase


class TokenAuthentication(AuthBase):
    def __init__(self, token):
        self.token = token

    def __call__(self, r):
        r.headers['Authorization'] = 'Bearer {token}'.format(token=self.token)
        return r



import json
from dataclasses import dataclass

import omitempty

from dnsimple.struct import Struct


@dataclass
class ZoneRecordUpdateInput(dict):
    """Represents the data to send tot the DNSimple API to update a zone record"""

    def __init__(self, name=None, content=None, ttl=None, priority=None, regions=None):
        dict.__init__(self, name=name, content=content, ttl=ttl, priority=priority, regions=regions)

    def to_json(self):
        omitted = omitempty(self)

        if self['name'] == '':
            omitted['name'] = ''

        return json.dumps(omitted)


@dataclass
class ZoneRecordInput(dict):
    """Represents the data to send to the DNSimple API to create a zone record"""

    def __init__(self, name, type, content, ttl=None, priority=None, regions=None):
        dict.__init__(self, name=name, type=type, content=content, ttl=ttl, priority=priority, regions=regions)

    def to_json(self):
        omitted = omitempty(self)

        if self['name'] == '':
            omitted['name'] = ''

        return json.dumps(omitted)

@dataclass
class ZoneRecord(Struct):
    """Represents a zone record"""
    id = None
    """The record ID in DNSimple"""
    zone_id = None
    """The associated zone ID in DNSimple"""
    parent_id = None
    """The ID of the parent record, if this record is dependent on another record."""
    type = None
    """The type of record, in uppercase."""
    name = None
    """The record name (without the domain name)."""
    content = None
    """The plain-text record content"""
    ttl = None
    """The TTL value"""
    priority = None
    """The priority value, if the type of record accepts a priority."""
    regions = None
    """The regions where the record is propagated. This is optional."""
    system_record = None
    """True if this is a system record created by DNSimple. System records are read-only."""
    created_at = None
    """When the record was created in DNSimple."""
    updated_at = None
    """When the record was last updated in DNSimple."""

    def __init__(self, data):
        super().__init__(data)



from dnsimple.response import Response
from dnsimple.struct import Contact


class Contacts(object):
    """
    Handles communication with the contact related methods of the DNSimple API.

    See https://developer.dnsimple.com/v2/contacts/
    """

    def __init__(self, client):
        self.client = client

    def list_contacts(self, account_id, page=None, per_page=None):
        """
        Lists the contacts in the account

        See https://developer.dnsimple.com/v2/contacts/#listContacts

        :param account_id: int
            The account id
        :param page: int
            The page to return (default: 1)
        :param per_page: int
            The number of entries to return per page (default: 30, maximum: 100)

        :return: dnsimple.Response
            The list of contacts
        """
        response = self.client.get(f'/{account_id}/contacts', page=page, per_page=per_page)
        return Response(response, Contact)

    def create_contact(self, account_id, contact):
        """
        Creates a new contact in the account

        See https://developer.dnsimple.com/v2/contacts/#createContact
        
        :param account_id: int
            The account id
        :param contact: dnsimple.struct.Contact
            The contact to be created

        :return: dnsimple.Response
            The newly created contact
        """
        response = self.client.post(f'/{account_id}/contacts', data=contact.to_json())
        return Response(response, Contact)

    def get_contact(self, account_id, contact_id):
        """
        Gets a contact from the account

        See https://developer.dnsimple.com/v2/contacts/#getContact

        :param account_id: int
            Then account id
        :param contact_id: int
            The contact id

        :return: dnsimple.Response
            The contact
        """
        response = self.client.get(f'/{account_id}/contacts/{contact_id}')
        return Response(response, Contact)

    def update_contact(self, account_id, contact_id, contact):
        """
        Updates a contact in the account

        See https://developer.dnsimple.com/v2/contacts/#updateContact

        :param account_id: int
            The account id
        :param contact_id: int
            The contact id
        :param contact: dnsimple.struct.Contact
            The contact attributes to be updated

        :return: dnsimple.Response
            The updated contact
        """
        response = self.client.patch(f'/{account_id}/contacts/{contact_id}', data=contact.to_json())
        return Response(response, Contact)

    def delete_contact(self, account_id, contact_id):
        """
        Deletes a contact from the account

        See https://developer.dnsimple.com/v2/contacts/#deleteContact

        :param account_id: int
            The account id
        :param contact_id: int
            The contact id

        :return: dnsimple.Response
            An empty response
        """
        response = self.client.delete(f'/{account_id}/contacts/{contact_id}')
        return Response(response)



import json
from dataclasses import dataclass

import omitempty

from dnsimple.extra import attach_attributes_to


@dataclass
class TemplateRecord(dict):
    """Represents a DNS record for a template in DNSimple"""

    id = None
    """The template record ID in DNSimple"""
    template_id = None
    """The template ID in DNSimple"""
    name = None
    """The template record name (without the domain name)"""
    content = None
    """The plain-text template record content"""
    ttl = None
    """The template record TTL value"""
    type = None
    """The type of template record, in uppercase"""
    priority = None
    """The priority value, if the type of template record accepts a priority"""
    created_at = None
    """When the template record was created in DNSimple"""
    updated_at = None
    """When the template record was last updated in DNSimple"""

    def __init__(self, data):
        attach_attributes_to(self, data)
        dict.__init__(self, id=self.id, template_id=self.template_id, name=self.name, content=self.content,
                      ttl=self.ttl, type=self.type, priority=self.priority, created_at=self.created_at,
                      updated_at=self.updated_at)

    def to_json(self):
        return json.dumps(omitempty(self))

    @classmethod
    def new(cls, name, type, content, ttl=None, priority=None):
        """
        Creates a new template record

        :param name: str
        :param type: str
        :param content: str
        :param ttl: int
        :param priority: int
        :return: TemplateRecord
        """
        return cls({'name': name, 'type': type, 'content': content, 'ttl': ttl, 'priority': priority})



from dnsimple.response import Response
from dnsimple.struct import Zone, ZoneDistribution, ZoneFile, ZoneRecord, BatchChangeZoneRecordsResponse


class Zones(object):
    """
    The Zones Service handles the zones endpoint of the DNSimple API.

    See https://developer.dnsimple.com/v2/zones
    """

    def __init__(self, client):
        self.client = client

    def activate_dns(self, account_id, zone_name):
        """
        Activates DNS resolution for the zone in the account.

        See https://developer.dnsimple.com/v2/zones/#activateZoneService

        :param account_id: int
            The account ID
        :param zone_name:
            The zone name

        :return: dnsimple.Response
        """
        response = self.client.put(f'/{account_id}/zones/{zone_name}/activation')
        return Response(response, Zone)

    def deactivate_dns(self, account_id, zone_name):
        """
        Deactivates DNS resolution for the zone in the account.

        See https://developer.dnsimple.com/v2/zones/#deactivateZoneService

        :param account_id: int
            The account ID
        :param zone_name:
            The zone name

        :return: dnsimple.Response
        """
        response = self.client.delete(f'/{account_id}/zones/{zone_name}/activation')
        return Response(response, Zone)

    def list_zones(self, account_id, filter=None, sort=None, page=None, per_page=None):
        """
        Lists the zones in the account.

        See https://developer.dnsimple.com/v2/zones/#listZones

        :param account_id: int
            The account ID
        :param sort: str
            Comma separated key-value pairs: the name of a field and the order criteria (asc for ascending and desc for
            descending).

            Possible sort criteria:
                - id: Sort zones by ID (i.e. 'id:asc')
                - name: Sort zones by name (alphabetical order) (i.e. 'name:desc')
        :param filter: dict
            Makes it possible to ask only for the exact subset of data that you you’re looking for.

            Possible filters:
                - name_like: Only include zones containing given string (i.e. {'name_like': 'example.com'} )
        :param page: int
            The page to return (default: 1)
        :param per_page: int
            The number of entries to return per page (default: 30, maximum: 100)

        :return: dnsimple.Response
            The list of zones requested
        """
        response = self.client.get(f'/{account_id}/zones', filter=filter, sort=sort, page=page, per_page=per_page)
        return Response(response, Zone)

    def get_zone(self, account_id, zone):
        """
        Gets a zone from the account

        See https://developer.dnsimple.com/v2/zones/#getZone

        :param account_id: int
            The account ID
        :param zone: str
            The zone name

        :return: dnsimple.Response
            The zone requested
        """
        response = self.client.get(f'/{account_id}/zones/{zone}')
        return Response(response, Zone)

    def get_zone_file(self, account_id, zone):
        """
        Gets a zone file from the account

        See https://developer.dnsimple.com/v2/zones/#getZoneFile

        :param account_id: int
            The account ID
        :param zone: str
            The zone name

        :return: dnsimple.Response
            The zone file requested
        """
        response = self.client.get(f'/{account_id}/zones/{zone}/file')
        return Response(response, ZoneFile)

    def check_zone_distribution(self, account_id, zone):
        """
        Checks if a zone change is fully distributed to all DNSimple name servers across the globe.

        WARNING: This feature can’t be tested in our Sandbox environment.

        See https://developer.dnsimple.com/v2/zones/#checkZoneDistribution

        :param account_id: int
            The account ID
        :param zone: str
            The zone name

        :return: dnsimple.Response
            The zone distribution
        """
        response = self.client.get(f'/{account_id}/zones/{zone}/distribution')
        return Response(response, ZoneDistribution)

    def list_records(self, account_id, zone, filter=None, sort=None, page=None, per_page=None):
        """
        Lists the zone records in the account

        See https://developer.dnsimple.com/v2/zones/records/#listZoneRecords

        :param account_id: int
            The account ID
        :param zone: str
            The zone name
        :param sort: str
            Comma separated key-value pairs: the name of a field and the order criteria (asc for ascending and desc for
            descending).

            Possible sort criteria:
                - id: Sort records by ID (i.e. 'id:asc')
                - name: Sort records by name (i.e. 'name:desc')
                - content: Sort records by content (i.e. 'content:asc')
                - type: Sort records by type (i.e. 'type:asc')
        :param filter: dict
            Makes it possible to ask only for the exact subset of data that you you’re looking for.

            Possible filters:
                - name_like: Only include records where the name contains the given string (i.e.{'name_like':'example'})
                - name: Only include records with name equal to the given string (i.e. {'name': 'example.com'})
                - type: Only include records with the record type equal to the given string (i.e. {'type': 'SOA'})
        :param page: int
            The page to return (default: 1)
        :param per_page: int
            The number of entries to return per page (default: 30, maximum: 100)

        :return: dnsimple.Response
            The list of zone records in the account
        """
        response = self.client.get(f'/{account_id}/zones/{zone}/records', filter=filter, sort=sort, page=page,
                                   per_page=per_page)
        return Response(response, ZoneRecord)

    def create_record(self, account_id, zone, record):
        """
        Create a record for the zone in the account

        See https://developer.dnsimple.com/v2/zones/records/#createZoneRecord

        :param account_id: int
            The account ID
        :param zone: str
            The zone name
        :param record: dnsimple.struct.ZoneRecordInput
            The data to send to create the zone record

        :return: dnsimple.Response
            The newly created zone record
        """
        response = self.client.post(f'/{account_id}/zones/{zone}/records', data=record.to_json())
        return Response(response, ZoneRecord)

    def get_record(self, account_id, zone, record_id):
        """
        Gets a zone record from the account

        See https://developer.dnsimple.com/v2/zones/records/#getZoneRecord

        :param account_id: int
            The account ID
        :param zone: str
            The zone name
        :param record_id: int
            The record ID

        :return: dnsimple.Response
            The zone record requested
        """
        response = self.client.get(f'/{account_id}/zones/{zone}/records/{record_id}')
        return Response(response, ZoneRecord)

    def update_record(self, account_id, zone, record_id, record):
        """
        Updates a zone record in the account.

        See https://developer.dnsimple.com/v2/zones/records/#updateZoneRecord

        :param account_id: int
            The account ID
        :param zone: str
            The zone name
        :param record_id: int
            The record ID
        :param record: dnsimple.struct.ZoneRecordUpdateInput
            The data to send to update the zone record
        :return: dnsimple.Response
            The updated zone record
        """
        response = self.client.patch(f'/{account_id}/zones/{zone}/records/{record_id}', data=record.to_json())
        return Response(response, ZoneRecord)

    def delete_record(self, account_id, zone, record_id):
        """
        Deletes a zone record from the account.

        WARNING: this cannot be undone.

        See https://developer.dnsimple.com/v2/zones/records/#deleteZoneRecord

        :param account_id: int
            The account ID
        :param zone: str
            The zone name
        :param record_id: int
            The record ID
        :return: dnsimple.Response
            An empty response
        """
        response = self.client.delete(f'/{account_id}/zones/{zone}/records/{record_id}')
        return Response(response)

    def check_zone_record_distribution(self, account_id, zone, record_id):
        """
        Checks if a zone record is fully distributed to all DNSimple name servers across the globe.

        WARNING: This feature can’t be tested in our Sandbox environment.

        See https://developer.dnsimple.com/v2/zones/records/#checkZoneRecordDistribution

        :param account_id: int
            The account ID
        :param zone: str
            The zone name
        :param record_id: int
            The record ID

        :return: dnsimple.Response
            The zone record distribution
        """
        response = self.client.get(f'/{account_id}/zones/{zone}/records/{record_id}/distribution')
        return Response(response, ZoneDistribution)

    def batch_change_records(self, account_id, zone, batch_change):
        """
        Batch change zone records in the account.

        See https://developer.dnsimple.com/v2/zones/records/#batchChangeZoneRecords

        :param account_id: int
            The account ID
        :param zone: str
            The zone name
        :param batch_change: dnsimple.struct.BatchChangeZoneRecordsInput
            The data to send to batch change zone records

        :return: dnsimple.Response
            The batch change zone records response
        """
        response = self.client.post(f'/{account_id}/zones/{zone}/batch', data=batch_change.to_json())
        return Response(response, BatchChangeZoneRecordsResponse)



import warnings

class DNSimpleException(Exception):
    """
    Root Exception raised for errors in the client.

    :param message: str
        The raw response from the server
    :param attribute_errors: dict
        A dict of attribute error
    :param http_response: requests.Response
        HTTP response object
    """

    def __init__(self, message=None, attribute_errors=None, http_response=None):
        self.message = message
        self.attribute_errors = attribute_errors
        self.status = None
        self.reason = None
        self.response = None

        if http_response is not None:
            self.reason = http_response.reason
            self.status = http_response.status_code
            self.response = http_response

    def __str__(self):
        """Custom error messages for exception"""
        error_message = "({0})\n"\
                        "Reason: {1}\n".format(self.status, self.reason)

        if self.response is not None:
            error_message += "HTTP response body: {0}\n".format(self.response.text)

        return error_message

    @property
    def errors(self):
        """Return the attribute errors"""
        warnings.warn("DEPRECATION WARNING: errors is deprecated, use attribute_errors instead.")
        return self.attribute_errors



import json
from dataclasses import dataclass

import omitempty

from dnsimple.extra import attach_attributes_to


@dataclass
class Contact(dict):
    id = None
    """The contact ID in DNSimple"""
    account_id = None
    """The associated account ID"""
    label = None
    """The label to represent the contact"""
    first_name = None
    """The contact first name"""
    last_name = None
    """The contact last name"""
    job_title = None
    """The contact's job title"""
    organization_name = None
    """The name of the organization in which the contact works"""
    address1 = None
    """The contact street address"""
    address2 = None
    """Apartment or suite number"""
    city = None
    """The city name"""
    state_province = None
    """The state or province name"""
    postal_code = None
    """The contact postal code"""
    country = None
    """The contact country (as a 2-character country code)"""
    phone = None
    """The contact phone number"""
    fax = None
    """The contact fax number (may be omitted)"""
    email = None
    """The contact email address"""
    created_at = None
    """When the contact was created in DNSimple"""
    updated_at = None
    """When the contact was last updated in DNSimple"""

    def __init__(self, data):
        attach_attributes_to(self, data)
        dict.__init__(self, label=self.label, first_name=self.first_name, last_name=self.last_name,
                      job_title=self.job_title, organization_name=self.organization_name, email=self.email,
                      phone=self.phone, fax=self.fax, address1=self.address1, address2=self.address2, city=self.city,
                      state_province=self.state_province, postal_code=self.postal_code, country=self.country)

    def to_json(self):
        return json.dumps(omitempty(self))

    @classmethod
    def new(cls, label=None, first_name=None, last_name=None, job_title=None, organization_name=None, email=None,
            phone=None, fax=None, address1=None, address2=None, city=None, state_province=None, postal_code=None,
            country=None):
        """
        Creates a new instance a contact.

        :param label: str
        :param first_name: str
        :param last_name: st
        :param job_title: str
        :param organization_name: str
        :param email: str
        :param phone: str
        :param fax: str
        :param address1: str
        :param address2: str
        :param city: str
        :param state_province: str
        :param postal_code: str
        :param country: str

        :return: Contact
        """
        return cls({'label': label, 'first_name': first_name, 'last_name': last_name, 'job_title': job_title,
                    'organization_name': organization_name, 'email': email, 'phone': phone, 'fax': fax,
                    'address1': address1, 'address2': address2, 'city': city, 'state_province': state_province,
                    'postal_code': postal_code, 'country': country})



from urllib.parse import urljoin

from requests import Request, Session
from requests.auth import HTTPBasicAuth

from dnsimple.extra import prepare_params
from dnsimple.service import Accounts, Billing, Domains, DnsAnalytics, Identity, Oauth, Zones, Registrar, Certificates, Tlds, Contacts, \
    Services, Templates, VanityNameServers, Webhooks
from dnsimple.token_authentication import TokenAuthentication
from dnsimple.version import version


class Client(object):
    """
    Client for the DNSimple API

    You can use this service to consume the services the DNSimple API
    offers. All requests have to be done authenticated. You can either
    use basic authentication (email and password combination) or an
    oauth token (see https://developer.dnsimple.com/v2/oauth/).

    service = Client(email='example-user@example.com', 'password=secret')
    user = service.identity.whoami().data.user

    service = Client(access_token='SuperSecretToken')
    account = service.identity.whoami().data.account

    For more information on how to use the DNSimple API refer
    to https://developer.dnsimple.com
    """

    __base_url = 'https://api.dnsimple.com'
    """URL to the production environment"""

    __sandbox_base_url = 'https://api.sandbox.dnsimple.com'
    """URL to the sandbox environment"""

    __api_version = 'v2'
    """Current API version"""

    __user_agent = 'dnsimple-python/{version}'
    """Default user agent for the dnsimple-python service"""

    def __init__(self,
                 email=None, password=None,
                 access_token=None,
                 base_url=None,
                 sandbox=False,
                 user_agent=None):
        """
        Initializes the service.

        You can choose which way you want to authenticate, either by email/password
        or using an OAUTH2 access token you got from the oauth 'dance'
        (see https://developer.dnsimple.com/v2/oauth/)

        :param email: str
            DNSimple email address for the account
        :param password: str
            DNSimple password for the account
        :param access_token: str
            Access token you got assigned during the oauth dance
        :param base_url: str
            A different base url you might want to use (i.e. for
            testing on the sandbox environment)
        :param sandbox: bool
            Set to true if you want to point the client to the DNSimple sandbox environment
        :param user_agent: str
            A customized 'User-Agent' header for the calls made to the DNSimple API
        """
        self._default_user_agent = self.__user_agent.format(version=version)
        self.user_agent = self._default_user_agent

        self.base_url = self.__base_url
        self.api_version = self.__api_version
        self.session = Session()
        self.__attach_services()

        self.headers = {
            'User-Agent': self.user_agent,
            'Accept': 'application/json',
            'Content-Type': 'application/json'
        }

        if user_agent is not None:
            self.__change_user_agent(user_agent)

        self.base_url = self.__sandbox_base_url if sandbox is True else self.__base_url
        self.__modify_default_base_url(base_url)
        self.__add_authentication_method(access_token, email, password)

    def get(self, path, sort=None, filter=None, params=None, page=None, per_page=None):
        """
        Sends a GET request to the DNSimple API

        :param path: str
            Relative path to the service endpoint
        :param sort: str
            How to sort the response (when requesting lists)
        :param filter: dict
            Filters to be used on the request (when requesting lists)
        :param params: dict
            Query params that will be added to the request
        :param page: int
            The page to return (default: 1)
        :param per_page: int
            The number of entries to return per page (default: 30, maximum: 100)

        :return: requests.Response
            The response to the request sent
        """
        if params is None:
            params = {}
        params = {**params, **{'page': page, 'per_page': per_page}}
        request = Request(method='GET', url=self.versioned(path), headers=self.headers, auth=self.auth,
                          params=prepare_params(sort, filter, params))
        response = self.session.send(request.prepare())
        return response

    def post(self, path, data=None):
        """
        Sends a POST request to the DNSimple API

        :param path:  str
            Relative path to the service endpoint
        :param data: dict
            The data payload to send to the endpoint
        :return: requests.Response
            The response object for the request sent
        """
        request = Request(method='POST', url=self.versioned(path), headers=self.headers, auth=self.auth, data=data)
        return self.session.send(request.prepare())

    def put(self, path, data=None):
        """
        Sends a PUT request to the DNSimple API

        :param path:  str
            Relative path to the service endpoint
        :param data: obj
            The data payload to send to the endpoint
        :return: requests.Response
            The response object for the request sent
        """
        request = Request(method='PUT', url=self.versioned(path), headers=self.headers, auth=self.auth, data=data)
        return self.session.send(request.prepare())

    def patch(self, path, data=None):
        """
        Sends a PATCH request to the DNSimple API

        :param path: str
            Relative path to the service endpoint
        :param data: dict
            The data payload to send to the endpoint
        :return: requests.Response
            The response object for the request sent
        """
        request = Request(method='PATCH', url=self.versioned(path), headers=self.headers, auth=self.auth, data=data)
        return self.session.send(request.prepare())

    def delete(self, path):
        """
        Sends a DELETE request to the DNSimple API

        :param path: str
            Relative path to the service endpoint
        :return: request.Response
            The response object for the request sent
        """
        request = Request(method='DELETE', url=self.versioned(path), headers=self.headers, auth=self.auth).prepare()
        return self.session.send(request)

    def versioned(self, path):
        """
        Returns the URL to the API including the version

        :param path: str
            The relative path to the API endpoint

        :return: str
            The URL to the API including the version
        """
        versioned_base_url = urljoin(self.base_url, self.api_version)
        return '{url}/{path}'.format(url=versioned_base_url, path=path.replace('/', '', 1))

    def __change_user_agent(self, custom_user_agent_name):
        """
        Changes the User agent to a custom name.

        Note that the custom name you use will be prepended to the default
        user agent for the python service.

        :param custom_user_agent_name: str
            The custom name you want to use as your user agent
        """
        self.user_agent = '{custom_name} {user_agent}'.format(
            custom_name=custom_user_agent_name,
            user_agent=self.__user_agent.format(version=version))
        self.headers['User-Agent'] = self.user_agent

    def __modify_default_base_url(self, base_url):
        if base_url is not None:
            self.base_url = base_url

    def __add_authentication_method(self, access_token, email, password):
        if access_token is not None:
            self.auth = TokenAuthentication(access_token)
        elif email is not None and password is not None:
            self.auth = HTTPBasicAuth(email, password)

    def __attach_services(self):
        self.accounts = Accounts(self)
        self.billing = Billing(self)
        self.certificates = Certificates(self)
        self.contacts = Contacts(self)
        self.dns_analytics = DnsAnalytics(self)
        self.domains = Domains(self)
        self.identity = Identity(self)
        self.oauth = Oauth(self)
        self.registrar = Registrar(self)
        self.services = Services(self)
        self.templates = Templates(self)
        self.tlds = Tlds(self)
        self.vanity_name_servers = VanityNameServers(self)
        self.webhooks = Webhooks(self)
        self.zones = Zones(self)



import json
from dataclasses import dataclass

import omitempty

from dnsimple.struct import Struct


class DomainRenewRequest(dict):
    """DomainRenewRequest represents the attributes you can pass to a renew API request."""
    def __init__(self, period=None, premium_price=None):
        dict.__init__(self, period=period, premium_price=premium_price)

    def to_json(self):
        return json.dumps(omitempty(self))


@dataclass
class DomainRenewal(Struct):

    """Represents the result of a domain renewal call."""
    id = None
    """The domain registration ID in DNSimple"""
    domain_id = None
    """The associated domain ID"""
    state = None
    """The state of the renewal"""
    period = None
    """The number of years the domain was registered for"""
    created_at = None
    """When the domain renewal was created in DNSimple"""
    updated_at = None
    """When the domain renewal was last updated in DNSimple"""

    def __init__(self, data):
        super().__init__(data)



from dnsimple.response import Response
from dnsimple.struct import Webhook


class Webhooks(object):
    def __init__(self, client):
        self.client = client

    def list_webhooks(self, account_id, sort=None):
        """
        List the webhooks in the account

        See https://developer.dnsimple.com/v2/webhooks/#listWebhooks

        :param account_id: int
            The account id
        :param sort: str
            Comma separated key-value pairs: the name of a field and the order criteria (asc for ascending and desc for
            descending).

            Possible sort criteria:
                - id: Sort webhooks by ID (i.e. 'id:asc')

        :return: dnsimple.Response
            The list of webhooks
        """
        response = self.client.get(f'/{account_id}/webhooks', sort=sort)
        return Response(response, Webhook)

    def create_webhook(self, account_id, webhook):
        """
        Creates a webhook in the account

        See https://developer.dnsimple.com/v2/webhooks/#createWebhook

        :param account_id: int
            The account id
        :param webhook: dnsimple.struct.Webhook
            Then webhook attributes

        :return: dnsimple.Response
            The newly created webhook
        """
        response = self.client.post(f'/{account_id}/webhooks', data=webhook.to_json())
        return Response(response, Webhook)

    def get_webhook(self, account_id, webhook):
        """
        Gets a webhook from the account

        See https://developer.dnsimple.com/v2/webhooks/#getWebhook

        :param account_id: int
            The account id
        :param webhook: int/str
            The webhook id

        :return: dnsimple.Response
            The webhook
        """
        response = self.client.get(f'/{account_id}/webhooks/{webhook}')
        return Response(response, Webhook)

    def delete_webhook(self, account_id, webhook):
        """
        Deletes a webhook from the account

        See https://developer.dnsimple.com/v2/webhooks/#deleteWebhook

        :param account_id: int
            The account id
        :param webhook: int/str
            The webhook id

        :return: dnsimple.Response
            An empty response
        """
        response = self.client.delete(f'/{account_id}/webhooks/{webhook}')
        return Response(response)



from dnsimple.response import Response
from dnsimple.struct import VanityNameServer


class VanityNameServers(object):
    def __init__(self, client):
        self.client = client

    def enable_vanity_name_servers(self, account_id, domain):
        """
        Enables Vanity Name Servers for the domain

        See https://developer.dnsimple.com/v2/vanity/#enableVanityNameServers

        :param account_id: int
            The account id
        :param domain: int/str
            The domain name or id
        :return: dnsimple.Response
            The vanity name server list
        """
        response = self.client.put(f'/{account_id}/vanity/{domain}')
        return Response(response, VanityNameServer)

    def disable_vanity_name_servers(self, account_id, domain):
        """
        Disables Vanity Name Servers for the domain

        See https://developer.dnsimple.com/v2/vanity/#disableVanityNameServers

        :param account_id: int
            The account id
        :param domain: int/str
            The domain name or id

        :return: dnsimple.Response
            An empty response
        """
        response = self.client.delete(f'/{account_id}/vanity/{domain}')
        return Response(response)



from dataclasses import dataclass

from dnsimple.struct import Struct


@dataclass
class Tld(Struct):
    """Represents a TLD in DNSimple"""

    tld = None
    """The TLD in DNSimple"""
    tld_type = None
    """The TLD type"""
    whois_privacy = None
    """True if Whois Privacy Protection is available"""
    auto_renew_only = None
    """True if TLD requires use of auto-renewal for renewals"""
    idn = None
    """True if IDN is available"""
    minimum_registration = None
    """The minimum registration period, in years"""
    registration_enabled = None
    """True if DNSimple supports registrations for this TLD"""
    renewal_enabled = None
    """True if DNSimple supports renewals for this TLD"""
    transfer_enabled = None
    """True if DNSimple supports inbound transfers for this TLD"""
    dnssec_interface_type = None
    """Type of data interface required for DNSSEC for this TLD"""

    def __init__(self, data):
        super().__init__(data)


@dataclass
class TldExtendedAttributeOption(Struct):
    """Represents a single option you can assign to an extended attributes."""

    title = None
    """The option name"""
    value = None
    """The option value"""
    description = None
    """A long description of the option"""

    def __init__(self, data):
        super().__init__(data)


@dataclass
class TldExtendedAttribute(Struct):
    """Represents an extended attributes supported or required by a specific TLD."""

    name = None
    """The extended attribute name"""
    description = None
    """A description of the extended attribute"""
    required = None
    """Boolean indicating if the extended attribute is required"""
    options = None
    """The list of options with possible values for the extended attribute"""

    def __init__(self, data):
        super().__init__(data)
        self.options = list(map(TldExtendedAttributeOption, self.options))



from dataclasses import dataclass

from dnsimple.struct import Struct


@dataclass
class ZoneFile(Struct):
    """Represents a zone file"""

    zone = None
    """The zone file content"""

    def __init__(self, data):
        super().__init__(data)



from dataclasses import dataclass

from dnsimple.struct import Struct


@dataclass
class Zone(Struct):
    """
    Represents a Zone

    See https://developer.dnsimple.com/v2/zones
    """

    id = None
    """The zone ID in DNSimple"""
    account_id = None
    """The associated account ID in DNSimple"""
    name = None
    """The zone name"""
    reverse = None
    """True if the zone is a reverse zone"""
    secondary = None
    """True if the zone is a secondary zone"""
    last_transferred_at = None
    """When the zone was last transferred"""
    active = None
    """True if the zone is active"""
    created_at = None
    """When the zone was created in DNSimple"""
    updated_at = None
    """When the zone was last updated in DNSimple"""

    def __init__(self, data):
        super().__init__(data)



from dnsimple.response import Response
from dnsimple.struct import Template, TemplateRecord


class Templates(object):
    def __init__(self, client):
        self.client = client

    def list_templates(self, account_id, sort=None, page=None, per_page=None):
        """
        Lists the templates in the account

        See https://developer.dnsimple.com/v2/templates/#listTemplates

        :param account_id: int
            The account id
        :param sort: str
            Comma separated key-value pairs: the name of a field and the order criteria (asc for ascending and desc for
            descending).

            Possible sort criteria:
                - id: Sort templates by ID (i.e. 'id:asc')
                - name: Sort templates by name (alphabetical order) (i.e. 'name:desc')
                - sid: Sort templates by sid (i.e. 'sid:asc')
        :param page: int
            The page to return (default: 1)
        :param per_page: int
            The number of entries to return per page (default: 30, maximum: 100)

        :return: dnsimple.Response
            The templates in the account
        """
        response = self.client.get(f'/{account_id}/templates', sort=sort, page=page, per_page=per_page)
        return Response(response, Template)

    def create_template(self, account_id, template_attributes):
        """
        Creates a template in the account

        See https://developer.dnsimple.com/v2/templates/#createTemplate

        :param account_id: int
            The account id
        :param template_attributes: dnsimple.struct.Template
            The template attributes
        :return: dnsimple.Response
            The newly created template
        """
        response = self.client.post(f'/{account_id}/templates', data=template_attributes.to_json())
        return Response(response, Template)

    def get_template(self, account_id, template):
        """
        Gets the template with the specified id or sid

        See https://developer.dnsimple.com/v2/templates/#getTemplate

        :param account_id: int
            The account id
        :param template: int/str
            The template sid or id

        :return: dnsimple.Response
            The template requested
        """
        response = self.client.get(f'/{account_id}/templates/{template}')
        return Response(response, Template)

    def update_template(self, account_id, template, template_attributes):
        """
        Updates a template with the provided data

        See https://developer.dnsimple.com/v2/templates/#updateTemplate

        :param account_id: int
            The account id
        :param template: int/str
            The template id or sid
        :param template_attributes: dnsimple.struct.Template
            The template attributes to update the template with

        :return: dnsimple.Response
            The updated template
        """
        response = self.client.patch(f'/{account_id}/templates/{template}', data=template_attributes.to_json())
        return Response(response, Template)

    def delete_template(self, account_id, template):
        """
        Deletes a template from the account

        WARNING: This cannot be undone

        See https://developer.dnsimple.com/v2/templates/#deleteTemplate

        :param account_id: int
            The account id
        :param template: int/str
            The template id or sid

        :return: dnsimple.Response
            An empty response
        """
        response = self.client.delete(f'/{account_id}/templates/{template}')
        return Response(response)

    def list_template_records(self, account_id, template, sort=None, page=None, per_page=None):
        """
        Lists the records in the template

        See https://developer.dnsimple.com/v2/templates/records/#listTemplateRecords

        :param account_id: int
            The account id
        :param template: int/str
            The template id or sid
        :param sort: str
            Comma separated key-value pairs: the name of a field and the order criteria (asc for ascending and desc for
            descending).

            Possible sort criteria:
                - id: Sort template records by ID (i.e. 'id:asc')
                - name: Sort template records by name (i.e. 'name:desc')
                - content: Sort template records by content (i.e. 'content:asc')
                - type: Sort template records by type (i.e. 'type:desc')
        :param page: int
            The page to return (default: 1)
        :param per_page: int
            The number of entries to return per page (default: 30, maximum: 100)

        :return: dnsimple.Response
            The list of template records
        """
        response = self.client.get(f'/{account_id}/templates/{template}/records', sort=sort, page=page, per_page=per_page)
        return Response(response, TemplateRecord)

    def create_template_record(self, account_id, template, template_record_attributes):
        """
        Creates a record in the template

        See https://developer.dnsimple.com/v2/templates/records/#createTemplateRecord

        :param account_id: int
            The account id
        :param template: int/str
            The template id or sid
        :param template_record_attributes: dnsimple.struct.TemplateRecord
            The template record attributes

        :return: dnsimple.Response
            The newly created template record
        """
        response = self.client.post(f'/{account_id}/templates/{template}/records', data=template_record_attributes.to_json())
        return Response(response, TemplateRecord)

    def get_template_record(self, account_id, template, record_id):
        """
        Gets a record from the template

        See https://developer.dnsimple.com/v2/templates/records/#getTemplateRecord

        :param account_id: int
            The account id
        :param template: int/str
            The template id or sid
        :param record_id: int
            The template record id

        :return: dnsimple.Response
            The template record
        """
        response = self.client.get(f'/{account_id}/templates/{template}/records/{record_id}')
        return Response(response, TemplateRecord)

    def delete_template_record(self, account_id, template, record_id):
        """
        Deletes a record from the template.

        WARNING: this cannot be undone.

        See https://developer.dnsimple.com/v2/templates/records/#deleteTemplateRecord

        :param account_id: int
            The account id
        :param template: int/str
            The template id or sid
        :param record_id: int
            The template record id

        :return: dnsimple.Response
            An empty response
        """
        response = self.client.delete(f'/{account_id}/templates/{template}/records/{record_id}')
        return Response(response)

    def apply_template(self, account_id, domain, template):
        """
        Applies a template to the domain

        See https://developer.dnsimple.com/v2/templates/domains/#applyTemplateToDomain

        :param account_id: int
            The account id
        :param domain: int/str
            The domain name or id
        :param template: int/str
            The template id or sid

        :return: dnsimple.Response
            An empty response
        """
        response = self.client.post(f'/{account_id}/domains/{domain}/templates/{template}')
        return Response(response)



from dataclasses import dataclass

from dnsimple.struct import Struct


@dataclass
class DnsAnalytics(Struct):
    """
    Represents DNS Analytics information.

    See https://developer.dnsimple.com/v2/dns-analytics/
    """

    volume = None
    """The volume"""
    zone_name = None
    """The zone name"""
    date = None
    """The date"""

    def __init__(self, data):
        super().__init__(data)



from dnsimple.response import Response
from dnsimple.struct import Account


class Accounts(object):
    """
    Lists the accounts the authenticated entity has access to.

    See http://developer.dnsimple.com/v2/accounts
    """
    def __init__(self, client):
        self.client = client

    def list_accounts(self):
        """
         Lists the accounts the current authenticated entity has access to.

        :return: dnsimple.Response
            The response containing the list of accounts.
        """
        response = self.client.get('/accounts')
        return Response(response, Account)



from dataclasses import dataclass

from dnsimple.struct import Struct


@dataclass
class AccessToken(Struct):
    """
    AccessToken represents a DNSimple Oauth access token.

    See https://developer.dnsimple.com/v2/oauth/
    """

    account_id = None
    """The account ID in DNSimple this token belongs to."""
    token_type = None
    """The token type."""
    access_token = None
    """The token you can use to authenticate."""
    scope = None
    """The token scope (not used for now)."""

    def __init__(self, data):
        super().__init__(data)



from dnsimple.response import Response
from dnsimple.struct import Charge

class Billing(object):
    def __init__(self, client):
        self.client = client

    def list_charges(self, account: int, *, start_date=None, end_date=None, sort=None):
        """
        Lists the billing charges for the account.

        See https://developer.dnsimple.com/v2/billing/#listCharges

        :param account:
            The account id
        """
        response = self.client.get(f'/{account}/billing/charges', params={"start_date": start_date, "end_date": end_date, "sort": sort})
        return Response(response, Charge)



def return_list_of(cls, data):
    """
    Returns a list of instances of the class type found in the response.

    :param cls: class
        The class type we want to instantiate
    :param data: list
        The data containing the list of the dictionaries we want to return as objects
    :return:
        A list of objects of the type specified found in the response
    """
    return [cls(x) for x in data]


def return_list_with_headers_of(cls, rows, headers):
    """
    Returns a list of instances of the class type found in the response.

    :param cls: class
        The class type we want to instantiate
    :param rows: list
        The rows of with values we want to return as objects
    :param headers: list
        The name of the attributes in order expected in each row
    :return:
        A list of objects of the type specified found in the response
    """
    return [cls(dict(zip(headers, row))) for row in rows]


def attach_attributes_to(obj: object, data: object) -> object:
    """
    Attaches attributes to an object

    :param obj: object
        The object (class) to attach the attributes to
    :param data: dict
        A dictionary (taken from the 'data' attribute in the json payload coming from the response) with the
        attributes and their values.
    :return: None
    """
    for index, attribute in enumerate(list(data)):
        setattr(obj, str(attribute), data[attribute])


def prepare_params(sort=None, filter=None, params=None):
    """
    Prepares the different parameters we want to send as the query in a request

    :param sort: str
        The sorting (used in list commands issued to the DNSimple API)
    :param filter: dict
        The filtering (used in list commands issued to the DNSimple API)
    :param params: dict
        Any other params we want to send in the query
    :return:
    """
    if params is None:
        params = {}
    if sort is not None:
        params = {**params, **{'sort': sort}}
    if filter is not None:
        params = {**params, **filter}
    return params



import json
from dataclasses import dataclass

import omitempty

from dnsimple.extra import attach_attributes_to


@dataclass
class Template(dict):
    """Represents a template in DNSimple"""

    id = None
    """The template ID in DNSimple"""
    account_id = None
    """The associated account ID"""
    name = None
    """The template name"""
    sid = None
    """The string ID for the template"""
    description = None
    """The template description"""
    created_at = None
    """When the template was created in DNSimple"""
    updated_at = None
    """When the template was last updated in DNSimple"""

    def __init__(self, data):
        attach_attributes_to(self, data)
        dict.__init__(self, id=self.id, account_id=self.account_id, name=self.name, sid=self.sid,
                      description=self.description, created_at=self.created_at, updated_at=self.updated_at)

    def to_json(self):
        return json.dumps(omitempty(self))

    @classmethod
    def new(cls, name, sid, description):
        """
        Creates a new template

        :param name: str
        :param sid: str
        :param description: str
        :return: Template
            The newly created template
        """
        return cls({'name': name, 'sid': sid, 'description': description})



from dataclasses import dataclass

from dnsimple.struct import Struct


@dataclass
class Domain(Struct):
    """
    Represents a Domain

    See https://developer.dnsimple.com/v2/domains/
    """

    id = None
    """The domain ID in DNSimple"""
    account_id = None
    """The associated account ID in DNSimple"""
    registrant_id = None
    """The associated registrant (contact) ID in DNSimple"""
    name = None
    """The domain name"""
    unicode_name = None
    """The domain unicode name"""
    state = None
    """The domain state"""
    auto_renew = False
    """True if the domain is set to auto-renew, false otherwise"""
    private_whois = False
    """True if the domain WHOIS privacy is enabled, false otherwise"""
    expires_at = None
    """The date the domain will expire"""
    created_at = None
    """When the domain was created in DNSimple"""
    updated_at = None
    """When the domain was last updated in DNSimple"""

    def __init__(self, data):
        super().__init__(data)



from dataclasses import dataclass

from dnsimple.struct.account import Account
from dnsimple.struct.user import User


@dataclass
class Whoami(object):
    """
    Represents the structure holding a User and Account object.
    """
    account = None
    """The account, if present"""
    user = None
    """The user, if present"""

    def __init__(self, data):
        self.account = None if data['account'] is None else Account(data['account'])
        self.user = None if data['user'] is None else User(data['user'])



from dnsimple import DNSimpleException
from dnsimple.extra import attach_attributes_to, return_list_of, return_list_with_headers_of


class Response(object):
    """
    Wrapper holding the whole http response as well as the data returned
    by an call to an endpoint of the DNSimple API.

    """

    http_response = None
    """The raw HTTP response from the DNSimple API call"""
    headers = None
    """The HTTP headers"""
    rate_limit = None
    """The maximum number of requests you can perform per hour."""
    rate_limit_remaining = None
    """The number of requests remaining in the current rate limit window."""
    rate_limit_reset = None
    """The time at which the current rate limit window in Unix time format."""
    data = None
    """The data returned from the DNSimple API call wrapped inside an object"""
    pagination = None
    """The pagination information wrapped in a Pagination object"""

    def __init__(self, http_response, obj=None):
        """

        :param http_response: requests.Response
            The raw response from the server
        :param obj: object
            An object encapsulating the data (json) in the response

        :raises DNSimpleException
            When the server responds with an error code
        """
        if int(http_response.status_code) in range(400, 504):
            if http_response.text != '':
                message = http_response.json().get('message')
                attribute_errors = http_response.json().get('errors')
                raise DNSimpleException(message=message, attribute_errors=attribute_errors, http_response=http_response)

            raise DNSimpleException(http_response=http_response)

        self.__class__.http_response = http_response
        self.__class__.headers = self.__class__.http_response.headers
        self.__class__.rate_limit = int(self.__class__.headers['X-RateLimit-Limit'])
        self.__class__.rate_limit_remaining = int(self.__class__.headers['X-RateLimit-Remaining'])
        self.__class__.rate_limit_reset = int(self.__class__.headers['X-RateLimit-Reset'])

        self.add_data(obj, http_response)

        # TODO Check response body is not empty as workaround for bug where server sends empty response despite having Content-Type: application/json, causing parser to crash.
        if http_response.status_code != 204 and http_response.text != '':
            self.__class__.pagination = None if http_response.json().get('pagination') is None else Pagination(
                http_response.json().get('pagination'))

    def add_data(self, obj, http_response):
        raw_json = None

        if obj is not None:
            raw_json = http_response.json()
        if obj is None:
            self.__class__.data = None
        elif 'data' not in raw_json:
            self.__class__.data = obj(raw_json)
        elif type(raw_json.get('data')) is list:
            self.__class__.data = return_list_of(obj, raw_json.get('data'))
        elif type(raw_json.get('data')) is dict and type(raw_json.get('data').get('rows')) is list and type(raw_json.get('data').get('headers')) is list:
            self.__class__.data = return_list_with_headers_of(obj, raw_json.get('data').get('rows'),
                                                              raw_json.get('data').get('headers'))
        else:
            self.__class__.data = obj(raw_json.get('data'))


class Pagination(object):
    """
    The pagination object

    Any API endpoint that returns a list of items requires pagination. By default we will return 30 records from any
    listing endpoint.

    If an API endpoint returns a list of items, then it will include a pagination object that contains pagination
    information.

    See https://developer.dnsimple.com/v2/#pagination
    """

    current_page = None
    """The page currently returned (default: 1)"""
    per_page = None
    """The number of entries returned per page (default: 30)"""
    total_entries = None
    """The total number of entries available in the entire collection"""
    total_pages = 1
    """The total number of pages available given the current per_page value"""

    def __init__(self, data):
        attach_attributes_to(self, data)



from dataclasses import dataclass
from dnsimple.struct import Struct
import json
import omitempty


@dataclass
class RegistrantChange(Struct):
    id = None
    account_id = None
    contact_id = None
    domain_id = None
    state = None
    extended_attributes = None
    registry_owner_change = None
    irt_lock_lifted_by = None
    created_at = None
    updated_at = None

    def __init__(self, data):
        super().__init__(data)


@dataclass
class RegistrantChangeCheck(Struct):
    contact_id = None
    domain_id = None
    extended_attributes = None
    registry_owner_change = None

    def __init__(self, data):
        super().__init__(data)


@dataclass
class CreateRegistrantChangeInput(dict):
    def __init__(self, domain_id = None, contact_id = None, extended_attributes = None):
        super().__init__(domain_id=domain_id, contact_id=contact_id, extended_attributes=extended_attributes)

    def to_json(self):
        return json.dumps(omitempty(self))


@dataclass
class CheckRegistrantChangeInput(dict):
    domain_id = None
    contact_id = None

    def __init__(self, domain_id = None, contact_id = None):
        super().__init__(domain_id=domain_id, contact_id=contact_id)

    def to_json(self):
        return json.dumps(omitempty(self))



from dnsimple.service.accounts import Accounts
from dnsimple.service.billing import Billing
from dnsimple.service.certificates import Certificates
from dnsimple.service.contacts import Contacts
from dnsimple.service.domains import Domains
from dnsimple.service.dns_analytics import DnsAnalytics
from dnsimple.service.identity import Identity
from dnsimple.service.oauth import Oauth
from dnsimple.service.registrar import Registrar
from dnsimple.service.services import Services
from dnsimple.service.templates import Templates
from dnsimple.service.tlds import Tlds
from dnsimple.service.vanity_name_servers import VanityNameServers
from dnsimple.service.webhooks import Webhooks
from dnsimple.service.zones import Zones



import json
import warnings

from dnsimple.response import Response
from dnsimple.struct import DomainCheck, DomainPremiumPrice, DomainRegistration, DomainTransfer, DomainRenewal, DomainRestore, RegistrantChange, VanityNameServer, WhoisPrivacy, WhoisPrivacyRenewal, DomainPrice, CheckRegistrantChangeInput, CreateRegistrantChangeInput, RegistrantChangeCheck, DomainTransferLock


class Registrar(object):
    """
    RegistrarService handles communication with the registrar related methods of the DNSimple API.

    See https://developer.dnsimple.com/v2/registrar/
    """

    def __init__(self, client):
        self.client = client

    def check_domain(self, account_id, domain):
        """
        Checks whether a domain is available to be registered.

        See https://developer.dnsimple.com/v2/registrar/#checkDomain

        :param account_id: int
            The account ID
        :param domain: str
            The domain name

        :return: dnsimple.Response
            The domain check result
        """
        response = self.client.get(f'/{account_id}/registrar/domains/{domain}/check')
        return Response(response, DomainCheck)

    def get_domain_premium_price(self, account_id, domain, options=None):
        """
        DEPERECATED: Get the premium price for a domain.
        Use get_domain_prices

        See https://developer.dnsimple.com/v2/registrar/#getDomainPremiumPrice

        :param account_id: int
            The account ID
        :param domain: str
            The domain name
        :param options: dnsimple.struct.DomainPremiumPriceOptions
            Optional action between "registration", "renewal", and "transfer".
            If omitted, it defaults to "registration".

        :return: dnsimple.Response
            The domain premium price requested
        """

        warnings.warn("DEPRECATION WARNING: get_domain_premium_price is deprecated, use get_domain_prices instead.")

        if options is None:
            options = {}
        response = self.client.get(f'/{account_id}/registrar/domains/{domain}/premium_price', params=options)
        return Response(response, DomainPremiumPrice)

    def get_domain_prices(self, account_id, domain):
        """
        Get prices for a domain.

        https://developer.dnsimple.com/v2/registrar/#getDomainPrices

        :param account_id: int
            The account ID
        :param domain: str
            The domain name

        :return: dnsimple.Response
            The domain prices
        """
        response = self.client.get(f'/{account_id}/registrar/domains/{domain}/prices')
        return Response(response, DomainPrice)

    def get_domain_registration(self, account_id, domain, domain_registration):
        """
        Get the details of an existing domain registration.

        https://developer.dnsimple.com/v2/registrar/#getDomainRegistration

        :param account_id: int
            The account ID
        :param domain: str
            The domain name
        :param domain_registration: int
            The domain registration ID

        :return: dnsimple.Response
            The domain registration
        """
        response = self.client.get(f'/{account_id}/registrar/domains/{domain}/registrations/{domain_registration}')
        return Response(response, DomainRegistration)

    def get_domain_renewal(self, account_id, domain, domain_renewal):
        """
        Get the details of an existing domain renewal.

        https://developer.dnsimple.com/v2/registrar/#getDomainRenewal

        :param account_id: int
            The account ID
        :param domain: str
            The domain name
        :param domain_renewal: int
            The domain renewal ID

        :return: dnsimple.Response
            The domain renewal
        """
        response = self.client.get(f'/{account_id}/registrar/domains/{domain}/renewals/{domain_renewal}')
        return Response(response, DomainRenewal)

    def register_domain(self, account_id, domain, request):
        """
        Registers a domain

        See https://developer.dnsimple.com/v2/registrar/#registerDomain

        :param account_id: int
            The account ID
        :param domain: str
            The domain name
        :param request: dnsimple.struct.DomainRegistrationRequest
            The attributes you can pass to register the domain

        :return: dnsimple.Response
            The newly registered domain
        """
        response = self.client.post(f'/{account_id}/registrar/domains/{domain}/registrations', request.to_json())
        return Response(response, DomainRegistration)

    def transfer_domain(self, account_id, domain, request):
        """
        Starts the transfer of a domain to DNSimple.

        See https://developer.dnsimple.com/v2/registrar/#transferDomain

        :param account_id: int
            The account ID
        :param domain: str
            The domain name
        :param request: dnsimple.struct.DomainTransferRequest
            The attributes you can pass to transfer the domain

        :return: dnsimple.Response
            The domain transfer
        """
        response = self.client.post(f'/{account_id}/registrar/domains/{domain}/transfers', request.to_json())
        return Response(response, DomainTransfer)

    def get_domain_transfer(self, account_id, domain, domain_transfer):
        """
        Retrieves the details of an existing domain transfer.

        See https://developer.dnsimple.com/v2/registrar/#getDomainTransfer

        :param account_id: int
            The account ID
        :param domain: str
            The domain name
        :param domain_transfer: int
            The domain transfer id

        :return: dnsimple.Response
            The details of an existing domain transfer
        """
        response = self.client.get(f'/{account_id}/registrar/domains/{domain}/transfers/{domain_transfer}')
        return Response(response, DomainTransfer)

    def cancel_domain_transfer(self, account_id, domain, domain_transfer):
        """
        Cancels an in progress domain transfer.

        See https://developer.dnsimple.com/v2/registrar/#cancelDomainTransfer

        :param account_id: int
            The account ID
        :param domain: str
            The domain name
        :param domain_transfer: int
            The domain transfer id

        :return: dnsimple.Response
            The details of the domain transfer
        """
        response = self.client.delete(f'/{account_id}/registrar/domains/{domain}/transfers/{domain_transfer}')
        return Response(response, DomainTransfer)

    def renew_domain(self, account_id, domain, request):
        """
        Renew a domain name already registered with DNSimple.

        Your account must be active for this command to complete successfully. You will be automatically charged the
        renewal fee upon successful renewal, so please be careful with this command.

        See https://developer.dnsimple.com/v2/registrar/#renewDomain

        :param account_id: int
            The account ID
        :param domain: str
            The domain name
        :param request: dnsimple.struct.DomainRenewRequest
            The renewal options

        :return: dnsimple.Request
            The domain renewal
        """
        response = self.client.post(f'/{account_id}/registrar/domains/{domain}/renewals', request.to_json())
        return Response(response, DomainRenewal)

    def transfer_domain_out(self, account_id, domain):
        """
        Prepare a domain for transferring out. This will unlock a domain and send the authorization code to the
        domain’s administrative contact.

        See https://developer.dnsimple.com/v2/registrar/#authorizeDomainTransferOut

        :param account_id: int
            The account ID
        :param domain: str
            The domain name

        :return: dnsimple.Response
            An empty response
        """
        response = self.client.post(f'/{account_id}/registrar/domains/{domain}/authorize_transfer_out')
        return Response(response)

    def get_domain_delegation(self, account_id, domain):
        """
        List name servers for the domain in the account.

        See https://developer.dnsimple.com/v2/registrar/delegation/#getDomainDelegation

        :param account_id: int
            The account ID
        :param domain: str
            The domain name

        :return: dnsimple.Response
            The list of name servers
        """
        response = self.client.get(f'/{account_id}/registrar/domains/{domain}/delegation')
        return Response(response, str)

    def change_domain_delegation(self, account_id, domain, new_delegation):
        """
        Update name servers for the domain in the account

        See https://developer.dnsimple.com/v2/registrar/delegation/#changeDomainDelegation

        :param account_id: int
            The account ID
        :param domain: str
            The domain name
        :param new_delegation: str[]
            The list of name servers

        :return: dnsimple.Response
            The list of name servers
        """
        response = self.client.put(f'/{account_id}/registrar/domains/{domain}/delegation', data=json.dumps(new_delegation))
        return Response(response, str)

    def change_domain_delegation_to_vanity(self, account_id, domain, new_delegation):
        """
        Delegate to vanity name servers

        WARNING: This method required the vanity name servers feature, that is only available for certain plans.
                 If the feature is not enabled, you will receive an HTTP 412 response code.

        See https://developer.dnsimple.com/v2/registrar/delegation/#changeDomainDelegationToVanity

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id
        :param new_delegation: str[]
            A list of name server names as strings.

        :return: dnsimple.Response
            The list of name servers
        """
        response = self.client.put(f'/{account_id}/registrar/domains/{domain}/delegation/vanity', data=json.dumps(new_delegation))
        return Response(response, VanityNameServer)

    def change_domain_delegation_from_vanity(self, account_id, domain):
        """
        Delegate from name servers

        WARNING: This method required the vanity name servers feature, that is only available for certain plans.
                 If the feature is not enabled, you will receive an HTTP 412 response code.

        See https://developer.dnsimple.com/v2/registrar/delegation/#changeDomainDelegationFromVanity

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id

        :return: dnsimple.Response
            An empty response
        """
        response = self.client.delete(f'/{account_id}/registrar/domains/{domain}/delegation/vanity')
        return Response(response)

    def enable_domain_auto_renewal(self, account_id, domain):
        """
        Enables auto renewal for the domain.

        See https://developer.dnsimple.com/v2/registrar/auto-renewal/#enableDomainAutoRenewal

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id

        :return: dnsimple.Response
            An empty response
        """
        response = self.client.put(f'/{account_id}/registrar/domains/{domain}/auto_renewal')
        return Response(response)

    def disable_domain_auto_renewal(self, account_id, domain):
        """
        Disables auto renewal for the domain.

        See https://developer.dnsimple.com/v2/registrar/auto-renewal/#disableDomainAutoRenewal

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id

        :return: dnsimple.Response
            An empty response
        """
        response = self.client.delete(f'/{account_id}/registrar/domains/{domain}/auto_renewal')
        return Response(response)

    def get_whois_privacy(self, account_id, domain):
        """
        Get the WHOIS privacy details for a domain

        See https://developer.dnsimple.com/v2/registrar/whois-privacy/#getWhoisPrivacy

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id

        :return: dnsimple.Response
            The whois privacy
        """
        response = self.client.get(f'/{account_id}/registrar/domains/{domain}/whois_privacy')
        return Response(response, WhoisPrivacy)

    def enable_whois_privacy(self, account_id, domain):
        """
        Enable WHOIS privacy

        Note that if the WHOIS privacy is not purchased for the domain, enabling WHOIS privacy will cause the service
        to be purchased for a period of 1 year.

        If WHOIS privacy was previously purchased and disabled, then calling this will enable the WHOIS privacy.

        See https://developer.dnsimple.com/v2/registrar/whois-privacy/#enableWhoisPrivacy

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id

        :return: dnsimple.Response
            The whois privacy
        """
        response = self.client.put(f'/{account_id}/registrar/domains/{domain}/whois_privacy')
        return Response(response, WhoisPrivacy)

    def disable_whois_privacy(self, account_id, domain):
        """
        Disable WHOIS privacy

        Note that if the WHOIS privacy is not purchased for the domain, this method will do nothing.

        If WHOIS privacy was previously purchased and enabled, then calling this will disable the WHOIS privacy.

        See https://developer.dnsimple.com/v2/registrar/whois-privacy/#disableWhoisPrivacy

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id

        :return: dnsimple.Response
            The whois privacy
        """
        response = self.client.delete(f'/{account_id}/registrar/domains/{domain}/whois_privacy')
        return Response(response, WhoisPrivacy)

    def renew_whois_privacy(self, account_id, domain):
        """
        Renew WHOIS privacy

        See https://developer.dnsimple.com/v2/registrar/whois-privacy/#renewWhoisPrivacy

        :param account_id: int
            The account ID
        :param domain: int/str
            The domain name or id

        :return: dnsimple.Response
            The whois privacy renewal
        """
        response = self.client.post(f'/{account_id}/registrar/domains/{domain}/whois_privacy')
        return Response(response, WhoisPrivacyRenewal)

    def list_registrant_changes(self, account: int, options=None):
        """
        List registrant changes in the account.

        See https://developer.dnsimple.com/v2/registrar/#listRegistrantChanges

        :param account:
            The account id
        :param options:
            Optional query parameters to filter and sort the results.
        """
        response = self.client.get(f"/{account}/registrar/registrant_changes", params=options)
        return Response(response, RegistrantChange)

    def create_registrant_change(
        self, account: int, input: CreateRegistrantChangeInput
    ):
        """
        Start registrant change.

        See https://developer.dnsimple.com/v2/registrar/#createRegistrantChange

        :param account:
            The account id
        """
        response = self.client.post(f"/{account}/registrar/registrant_changes", data=input.to_json())
        return Response(response, RegistrantChange)

    def check_registrant_change(
        self, account: int, input: CheckRegistrantChangeInput
    ):
        """
        Retrieves the requirements of a registrant change.

        See https://developer.dnsimple.com/v2/registrar/#checkRegistrantChange

        :param account:
            The account id
        """
        response = self.client.post(f"/{account}/registrar/registrant_changes/check", data=input.to_json())
        return Response(response, RegistrantChangeCheck)

    def get_registrant_change(self, account: int, registrantchange: int):
        """
        Retrieves the details of an existing registrant change.

        See https://developer.dnsimple.com/v2/registrar/#getRegistrantChange

        :param account:
            The account id
        :param registrantchange:
            The registrant change id
        """
        response = self.client.get(
            f"/{account}/registrar/registrant_changes/{registrantchange}"
        )
        return Response(response, RegistrantChange)

    def delete_registrant_change(self, account: int, registrantchange: int):
        """
        Cancel an ongoing registrant change from the account.

        See https://developer.dnsimple.com/v2/registrar/#deleteRegistrantChange

        :param account:
            The account id
        :param registrantchange:
            The registrant change id
        """
        response = self.client.delete(
            f"/{account}/registrar/registrant_changes/{registrantchange}"
        )
        return Response(
            response,
        )

    def get_domain_transfer_lock(self, account, domain):
        """
        Gets the transfer lock status for a domain.

        See https://developer.dnsimple.com/v2/registrar/#getDomainTransferLock

        :param account:
            The account id
        :param domain:
            The domain name or id
        """
        response = self.client.get(f'/{account}/registrar/domains/{domain}/transfer_lock')
        return Response(response, DomainTransferLock)

    def enable_domain_transfer_lock(self, account, domain):
        """
        Locks the domain to prevent unauthorized transfers.

        See https://developer.dnsimple.com/v2/registrar/#enableDomainTransferLock

        :param account:
            The account id
        :param domain:
            The domain name or id
        """
        response = self.client.post(f'/{account}/registrar/domains/{domain}/transfer_lock')
        return Response(response, DomainTransferLock)

    def disable_domain_transfer_lock(self, account, domain):
        """
        Unlocks the domain to allow domain transfers.

        See https://developer.dnsimple.com/v2/registrar/#disableDomainTransferLock

        :param account:
            The account id
        :param domain:
            The domain name or id
        """
        response = self.client.delete(f'/{account}/registrar/domains/{domain}/transfer_lock')
        return Response(response, DomainTransferLock)

    def restore_domain(self, account_id, domain, request):
        """
        Restores an expired domain name registered with DNSimple.

        Your account must be active for this command to complete successfully. You will be automatically charged the
        restore fee upon successful renewal, so please be careful with this command.

        See https://developer.dnsimple.com/v2/registrar/#restoreDomain

        :param account_id: int
            The account ID
        :param domain: str
            The domain name
        :param request: dnsimple.struct.DomainRestoreRequest
            The restore options

        :return: dnsimple.Request
            The domain restore
        """
        response = self.client.post(f'/{account_id}/registrar/domains/{domain}/restores', request.to_json())
        return Response(response, DomainRestore)

    def get_domain_restore(self, account_id, domain, domain_restore):
        """
        Get the details of an existing domain restore.

        https://developer.dnsimple.com/v2/registrar/#getDomainRestore

        :param account_id: int
            The account ID
        :param domain: str
            The domain name
        :param domain_restore: int
            The domain restore ID

        :return: dnsimple.Response
            The domain restore
        """
        response = self.client.get(f'/{account_id}/registrar/domains/{domain}/restores/{domain_restore}')
        return Response(response, DomainRestore)



from dnsimple.response import Response
from dnsimple.struct import DnsAnalytics as DnsAnalyticsStruct


class DnsAnalytics(object):
    """
    The DnsAnalytics Service handles the dns_analitycs endpoint of the DNSimple API.

    See http://developer.dnsimple.com/v2/dns-analytics
    """

    def __init__(self, client):
        self.client = client

    def query(self, account_id, filter=None, sort=None, params=None, page=None, per_page=None):
        """
        Retrieves the DNS analytics of the provided account.

        See https://developer.dnsimple.com/v2/dns-analytics/#queryDnsAnalytics

        :param account_id: int
            The account ID
        :param filter: dict
            Makes it possible to ask only for the exact subset of data that you’re looking for.

            Possible filters:
                - start_date: Only include results starting from the provided date in ISO8601 format. (i.e. {'start_date': '2024-02-01'} )
                - end_date: Only include results up to the provided date in ISO8601 format (i.e. {'end_date': '2024-03-01'} )

            These filters are optional and should be used together.
        :param params: dict
            Makes it change the shape of the obtained results

            Possible params:
                - groupings: Group results by the provided list of attributes separated by a comma. Possible attributes
                             are: date, zone_name. (i.e. {'groupings': 'date,zone_name'} )
        :param sort: str
            Comma separated key-value pairs: the name of a field and the order criteria (asc for ascending and desc for
            descending).

            Possible sort criteria:
                - date: Sort results by date (i.e. 'date:asc')
                - zone_name: Sort results by zone name (alphabetical order) (i.e. 'zone_name:desc')
                - volume: Sort results by volume (i.e. 'volume:asc')
        :param page: int
            The page to return (default: 1)
        :param per_page: int
            The number of entries to return per page (default: 1000, maximum: 10000)

        :return: dnsimple.Response
            A list of domains
        """
        response = self.client.get(f'/{account_id}/dns_analytics', sort=sort, filter=filter, params=params, page=page,
                                   per_page=per_page)
        return Response(response, DnsAnalyticsStruct)



from dataclasses import dataclass

from dnsimple.struct import Struct


@dataclass
class DomainPrice(Struct):
    """
    DomainPrice represents the registration, transfer, and renewal prices.
    """

    domain = None
    """The domain name"""
    premium = None
    """If the domain name is premium return True"""
    registartion_price = None
    """The domain's registation price"""
    renewal_price = None
    """The domain's renewal price"""
    transfer_price = None
    """The domain's transfer price"""
    def __init__(self, data):
        super().__init__(data)



import json
from dataclasses import dataclass

import omitempty

from dnsimple.struct import Struct


@dataclass
class DomainPushInput(dict):
    """Represents the data to send to the DNSimple API to initiate a push"""

    def __init__(self, new_account_email=None, contact_id=None):
        """
        Creates a new DomainPushInput

        :param new_account_email: str
            The target account email address (Required when initiating a push)
        :param contact_id: int
            A contact that belongs to the target DNSimple account. The contact will be used as new registrant for the
            domain, if the domain is registered with DNSimple (Required when accepting a push)
        """
        dict.__init__(self, new_account_email=new_account_email, contact_id=contact_id)

    def to_json(self):
        return json.dumps(omitempty(self))


@dataclass
class DomainPush(Struct):
    """Represents a domain push"""
    id = None
    """The domain push ID in DNSimple"""
    domain_id = None
    """The associated domain ID"""
    contact_id = None
    """The associated contact ID"""
    account_id = None
    """The associated account ID"""
    created_at = None
    """When the domain push was created in DNSimple"""
    updated_at = None
    """When the domain push was last updated in DNSimple"""
    accepted_at = None
    """When the domain push was accepted in DNSimple"""

    def __init__(self, data):
        super().__init__(data)



from dnsimple.response import Response
from dnsimple.struct import Service


class Services(object):
    def __init__(self, client):
        self.client = client

    def list_services(self, sort=None, page=None, per_page=None):
        """
        List the available one-click services.

        See https://developer.dnsimple.com/v2/services/#listServices

        :param sort: str
            Comma separated key-value pairs: the name of a field and the order criteria (asc for ascending and desc for
            descending).

            Possible sort criteria:
                - id: Sort services by ID (i.e. 'id:asc')
                - sid: Sort services by sid (i.e. 'sid:desc')
        :param page: int
            The page to return (default: 1)
        :param per_page: int
            The number of entries to return per page (default: 30, maximum: 100)

        :return: dnsimple.Response
            The list of services in DNSimple
        """
        response = self.client.get('/services', sort=sort, page=page, per_page=per_page)
        return Response(response, Service)

    def get_service(self, service_id):
        """
        Gets the service with specified ID

        See https://developer.dnsimple.com/v2/services/#getService

        :param service_id: int
            The service id

        :return: dnsimple.Response
            The service requested
        """
        response = self.client.get(f'/services/{service_id}')
        return Response(response, Service)

    def applied_services(self, account_id, domain, page=None, per_page=None):
        """
        List services applied to a domain

        See https://developer.dnsimple.com/v2/services/domains/#listDomainAppliedServices

        :param account_id: int
            The account id
        :param domain: int/str
            The domain name or id
        :param page: int
            The page to return (default: 1)
        :param per_page: int
            The number of entries to return per page (default: 30, maximum: 100)
        :return: dnsimple.Response
            The list of services applied to the domain
        """
        response = self.client.get(f'/{account_id}/domains/{domain}/services', page=page, per_page=per_page)
        return Response(response, Service)

    def apply_service(self, account_id, domain, service):
        """
        Applies a service to a domain

        See https://developer.dnsimple.com/v2/services/domains/#applyServiceToDomain

        :param account_id: int
            The account id
        :param domain: int/str
            The domain name or id
        :param service: int/str
            The service name or id
        :return: dnsimple.Response
            An empty response
        """
        response = self.client.post(f'/{account_id}/domains/{domain}/services/{service}')
        return Response(response)

    def unapply_service(self, account_id, domain, service):
        """
        Un-applies a service from a domain

        See https://developer.dnsimple.com/v2/services/domains/#unapplyServiceFromDomain

        :param account_id: int
            The account id
        :param domain: int/str
            The domain name or id
        :param service: int/str
            The service name or id
        :return: dnsimple.Response
            An empty response
        """
        response = self.client.delete(f'/{account_id}/domains/{domain}/services/{service}')
        return Response(response)



from dataclasses import dataclass

from dnsimple.struct import Struct


@dataclass
class WhoisPrivacy(Struct):
    """Represents a whois privacy in DNSimple"""

    id = None
    """The whois privacy ID in DNSimple"""
    domain_id = None
    """The associated domain ID"""
    expires_on = None
    """The date the whois privacy will expire on"""
    enabled = False
    """Whether the whois privacy is enabled for the domain"""
    created_at = None
    """When the whois privacy was created in DNSimple"""
    updated_at = None
    """When the whois privacy was last updated in DNSimple"""

    def __init__(self, data):
        super().__init__(data)


@dataclass
class WhoisPrivacyRenewal(Struct):
    """Represents a whois privacy renewal in DNSimple"""

    id = None
    """The whois privacy renewal ID in DNSimple"""
    domain_id = None
    """The associated domain ID"""
    whois_privacy_id = None
    """The associated whois privacy ID"""
    state = None
    """The whois Privacy order state"""
    expires_on = None
    """The date the whois Privacy will expire on"""
    enabled = False
    """Whether the whois Privacy is enabled for the domain"""
    created_at = None
    """When the whois Privacy was created in DNSimple"""
    updated_at = None
    """When the whois Privacy was last updated in DNSimple"""

    def __init__(self, data):
        super().__init__(data)



import json
from dataclasses import dataclass

import omitempty

from dnsimple.struct import Struct


class DomainRegistrationRequest(dict):
    """DomainRegisterRequest represents the attributes you can pass to a register API request."""

    def __init__(self, registrant_id, whois_privacy=False, auto_renew=False, extended_attributes=None,
                 premium_price=None):
        dict.__init__(self, registrant_id=registrant_id, whois_privacy=whois_privacy, auto_renew=auto_renew,
                      extended_attributes=extended_attributes, premium_price=premium_price)

    def to_json(self):
        return json.dumps(omitempty(self))


@dataclass
class DomainRegistration(Struct):
    """Represents the result of a domain registration call."""
    id = None
    """The domain registration ID in DNSimple"""
    domain_id = None
    """The associated domain ID"""
    registrant_id = None
    """The associated registrant (contact) ID"""
    period = None
    """The number of years the domain was registered for"""
    state = None
    """The state of the renewal"""
    auto_renew = False
    """True if the domain auto-renew was requested"""
    whois_privacy = False
    """True if the domain WHOIS privacy was requested"""
    created_at = None
    """When the domain renewal was created in DNSimple"""
    updated_at = None
    """When the domain renewal was last updated in DNSimple"""

    def __init__(self, data):
        super().__init__(data)



from dataclasses import dataclass

from dnsimple.struct import Struct


@dataclass
class User(Struct):
    """
    Represents a User.

    See https://developer.dnsimple.com/v2/identity/
    """

    id = None
    """The ID of the user in DNSimple"""
    email = None
    """The users email"""
    created_at = None
    """When the user was created in DNSimple"""
    updated_at = None
    """When the user was last updated in DNSimple"""

    def __init__(self, data):
        super().__init__(data)



from dataclasses import dataclass

from dnsimple.struct import Struct


@dataclass
class DomainPremiumPrice(Struct):
    """
    DomainPremiumPrice represents the premium price for a premium domain.
    """

    premium_price = None
    """The domain premium price"""
    action = None
    """The action (either registration, transfer or renewal)"""

    def __init__(self, data):
        super().__init__(data)


class DomainPremiumPriceOptions(dict):
    """
    DomainPremiumPriceOptions specifies the optional parameters you can provide to customize the
    dnsimple.services.Registrar.get_domain_premium_price method.
    """
    def __init__(self, action=None):
        dict.__init__(self, action=action)



from dnsimple.response import Response
from dnsimple.struct import Whoami


class Identity(object):
    """
    The Identity Service handles the identity (whoami) endpoint of the
    DNSimple API.

    See https://developer.dnsimple.com/v2/identity/
    """

    def __init__(self, client):
        """
        Initializes the service by passing the service.

        :param client: dnsimple.Client
            The service for the DNSimple API
        """
        self.client = client

    def whoami(self):
        """
        Retrieves the details about the current authenticated entity used to
        access the DNSimple API.

        :return: dnsimple.Response
            A response containing the details of the authenticated entity
        """
        response = self.client.get('/whoami')
        return Response(response, Whoami)



from dataclasses import dataclass

from dnsimple.struct import Struct


@dataclass
class Service(Struct):
    """Represents a Service in DNSimple"""

    id = None
    """The service ID in DNSimple"""
    name = None
    """The service name"""
    sid = None
    """A string ID for the service"""
    description = None
    """The service description"""
    setup_description = None
    """The service setup description"""
    requires_setup = None
    """Whether the service requires extra setup"""
    default_subdomain = None
    """The default subdomain where the service will be applied"""
    created_at = None
    """When the service was created in DNSimple"""
    updated_at = None
    """When the service was last updated in DNSimple"""
    settings = None
    """The array of settings to setup this service, if setup is required."""

    def __init__(self, data):
        super().__init__(data)
        self.settings = list(map(ServiceSetting, self.settings))


@dataclass
class ServiceSetting(Struct):
    """Represents a single group of settings for a DNSimple Service"""

    name = None
    """The setting name"""
    label = None
    """The setting label"""
    append = None
    """A suffix to be appended to the setting value"""
    description = None
    """The setting description"""
    example = None
    """An example of the setting value"""
    password = None
    """Whether the setting requires a password"""
    def __init__(self, data):
        super().__init__(data)



import json
from dataclasses import dataclass

import omitempty

from dnsimple.extra import attach_attributes_to


@dataclass
class Webhook(dict):
    """Represents a DNSimple webhook"""

    id = None
    """The contact ID in DNSimple"""
    url = None
    """The callback URL"""

    def __init__(self, data):
        attach_attributes_to(self, data)
        dict.__init__(self, id=self.id, url=self.url)

    def to_json(self):
        return json.dumps(omitempty(self))

    @classmethod
    def new(cls, url):
        return cls({'url': url})



from dataclasses import dataclass
from decimal import Decimal

from dnsimple.struct import Struct


@dataclass
class Charge(Struct):
    invoiced_at = None
    total_amount = None
    balance_amount = None
    reference = None
    state = None
    items = None

    def __init__(self, data):
        super().__init__(data)
        if self.total_amount is not None:
            self.total_amount = Decimal(self.total_amount)
        if self.balance_amount is not None:
            self.balance_amount = Decimal(self.balance_amount)


@dataclass
class ChargeItem(Struct):
    description = None
    amount = None
    product_id = None
    product_type = None
    product_reference = None

    def __init__(self, data):
        super().__init__(data)
        if self.amount is not None:
            self.amount = Decimal(self.amount)



import json
from dataclasses import dataclass

import omitempty

from dnsimple.struct import Struct
from dnsimple.struct.zone_record import ZoneRecord, ZoneRecordInput

BatchChangeZoneRecordsCreateInput = ZoneRecordInput


@dataclass
class BatchChangeZoneRecordsUpdateInput(dict):
    """Represents a zone record update input for a batch operation"""

    def __init__(self, id, name=None, content=None, ttl=None, priority=None, regions=None):
        dict.__init__(self, id=id, name=name, content=content, ttl=ttl, priority=priority, regions=regions)

    def to_json(self):
        omitted = omitempty(self)

        if self['name'] == '':
            omitted['name'] = ''

        return json.dumps(omitted)


@dataclass
class BatchChangeZoneRecordsDeleteInput(dict):
    """Represents a zone record deletion input for a batch operation"""

    def __init__(self, id):
        dict.__init__(self, id=id)


@dataclass
class BatchChangeZoneRecordsInput(dict):
    """Represents the data to send to the DNSimple API to make a batch change on the records of a zone

    All parameters are optional - you can perform creates only, updates only, deletes only,
    or any combination of the three operations.

    :param creates: List[BatchChangeZoneRecordsCreateInput] - Records to create (optional)
    :param updates: List[BatchChangeZoneRecordsUpdateInput] - Records to update (optional)
    :param deletes: List[BatchChangeZoneRecordsDeleteInput] - Records to delete (optional)
    """

    def __init__(self, creates=None, updates=None, deletes=None):
        data = {}
        if creates is not None:
            data['creates'] = creates
        if updates is not None:
            data['updates'] = updates
        if deletes is not None:
            data['deletes'] = deletes
        dict.__init__(self, **data)

    def to_json(self):
        result = {}

        if 'creates' in self and self['creates'] is not None:
            result['creates'] = [json.loads(item.to_json()) for item in self['creates']]

        if 'updates' in self and self['updates'] is not None:
            result['updates'] = [json.loads(item.to_json()) for item in self['updates']]

        if 'deletes' in self and self['deletes'] is not None:
            result['deletes'] = [omitempty(item) for item in self['deletes']]

        return json.dumps(result)


@dataclass
class BatchChangeZoneRecordsDeleteResponse(Struct):
    """Represents a deleted zone record in the batch change response"""
    id = None
    """The record ID that was deleted"""

    def __init__(self, data):
        super().__init__(data)


@dataclass
class BatchChangeZoneRecordsResponse(Struct):
    """Represents the response from batch changing zone records"""
    creates = None
    """List of created zone records"""
    updates = None
    """List of updated zone records"""
    deletes = None
    """List of deleted zone record IDs"""

    def __init__(self, data):
        super().__init__(data)
        if 'creates' in data and data['creates'] is not None:
            self.creates = [ZoneRecord(item) for item in data['creates']]
        if 'updates' in data and data['updates'] is not None:
            self.updates = [ZoneRecord(item) for item in data['updates']]
        if 'deletes' in data and data['deletes'] is not None:
            self.deletes = [BatchChangeZoneRecordsDeleteResponse(item) for item in data['deletes']]



from dataclasses import dataclass
from dnsimple.struct.struct import Struct

@dataclass
class DomainTransferLock(Struct):
    enabled = None

    def __init__(self, data):
        super().__init__(data)
